Package: Fragman
Version: 1.1.0
Date: 2026-05-20
Title: Fragment Analysis in R
Authors@R: c(
    person("Giovanny","Covarrubias-Pazaran", email="cova_ruber@live.com.mx", role=c("aut","cre"),
	comment=c(ORCID="0000-0002-7194-3837")),
	person("Luis", "Diaz-Garcia", role = "aut"),
  person("Brandon", "Schlautman", role = "aut"),
  person("Walter", "Salazar", role = "aut"),
  person("Juan", "Zalapa", role = "aut")
	)
Maintainer: Giovanny Covarrubias-Pazaran <cova_ruber@live.com.mx>
Description: Performs fragment analysis using genetic data coming from capillary electrophoresis machines. These are files with FSA extension which stands for FASTA-type file, and .txt files from Beckman CEQ 8000 system, both contain DNA fragment intensities read by machinery. In addition to visualization, it performs automatic scoring of SSRs (Sample Sequence Repeats; a type of genetic marker very common across the genome) and other type of PCR markers (standing for Polymerase Chain Reaction) in biparental populations such as F1, F2, BC (backcross), and diversity panels (collection of genetic diversity).
Depends: R(>= 2.10)
LazyData: true
License: GPL (>= 2)
NeedsCompilation: no
Packaged: 2026-05-24 02:15:32 UTC; giovannycovarrubias
Author: Giovanny Covarrubias-Pazaran [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7194-3837>),
  Luis Diaz-Garcia [aut],
  Brandon Schlautman [aut],
  Walter Salazar [aut],
  Juan Zalapa [aut]
Repository: CRAN
Date/Publication: 2026-05-24 16:10:08 UTC
Built: R 4.6.0; ; 2026-05-25 03:12:26 UTC; unix

Package: Fragman
Version: 1.1.0
Date: 2026-05-20
Title: Fragment Analysis in R
Authors@R: c(
    person("Giovanny","Covarrubias-Pazaran", email="cova_ruber@live.com.mx", role=c("aut","cre"),
	comment=c(ORCID="0000-0002-7194-3837")),
	person("Luis", "Diaz-Garcia", role = "aut"),
  person("Brandon", "Schlautman", role = "aut"),
  person("Walter", "Salazar", role = "aut"),
  person("Juan", "Zalapa", role = "aut")
	)
Maintainer: Giovanny Covarrubias-Pazaran <cova_ruber@live.com.mx>
Description: Performs fragment analysis using genetic data coming from capillary electrophoresis machines. These are files with FSA extension which stands for FASTA-type file, and .txt files from Beckman CEQ 8000 system, both contain DNA fragment intensities read by machinery. In addition to visualization, it performs automatic scoring of SSRs (Sample Sequence Repeats; a type of genetic marker very common across the genome) and other type of PCR markers (standing for Polymerase Chain Reaction) in biparental populations such as F1, F2, BC (backcross), and diversity panels (collection of genetic diversity).
Depends: R(>= 2.10)
LazyData: true
License: GPL (>= 2)
NeedsCompilation: no
Packaged: 2026-05-24 02:15:32 UTC; giovannycovarrubias
Author: Giovanny Covarrubias-Pazaran [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7194-3837>),
  Luis Diaz-Garcia [aut],
  Brandon Schlautman [aut],
  Walter Salazar [aut],
  Juan Zalapa [aut]
Repository: CRAN
Date/Publication: 2026-05-24 16:10:08 UTC
Built: R 4.5.2; ; 2026-05-25 01:11:00 UTC; unix

export(arrange.jm, best.layout, big.peaks.col, detect.ladder, find.ladder, 
       get.scores, homo.panel, homogenize.to.parentals, jm.conv, ladder.info.attach, 
       ladder.corrector, lapply_pb, letter.to.jm, num.to.lett, overview, overview2, 
       pullup, read.abif, reals, score.markers, saturate, separate, storing.inds, 
       threshs, transfft, transp)

importFrom("grDevices", "col2rgb", "colorRampPalette", "rgb", "rainbow","boxplot.stats")
  importFrom("graphics", "abline", "axis", "image", "layout", "legend",
             "lines", "plot", "points", "rect", "text", "locator",
             "par","grid")
  importFrom("stats", "cor", "fft", "lm", "median", "na.omit", "predict",
             "rnorm", "sd", "cov", "quantile")
  importFrom("utils", "combn", "setTxtProgressBar", "txtProgressBar","read.table")
  

## for s3 methods
export(plot.fsa_stored)

S3method(plot, fsa_stored)

detect.ladder <-
  function(stored, ind=1, ladder, channel.ladder=dim(stored[[1]])[2], ci.upp=1.96, ci.low=1.96, draw=TRUE, dev=50, warn=TRUE, init.thresh=250, sep.index=8, method="cor", avoid=1500, who="sample"){
    ##### stablish a minimum threshold
    x <- stored[[ind]][,channel.ladder]
    thresh=init.thresh
    roxy <- big.peaks.col(x, thresh) # find all peaks with that threshold
    nnn <- length(roxy$pos)
    while(nnn < length(ladder)){
      warning(paste("Warning: Your ladder for individual", who, "is bad, very low intensity was found, we are reducing the threshold 2x trying to find it, don't worry the analysis will keep going anyways"))
      thresh=thresh/2
      roxy <- big.peaks.col(x, thresh) # find all peaks with that threshold
      nnn <- length(roxy$pos)
    }
    ####
    #use the indicator peak at the beggining which is usually the tallest peak in the ladder
    qw <- which(roxy$hei == max(roxy$hei))
    roxy <- list(pos=roxy$pos[-qw], hei=roxy$hei[-qw])
    # get rid of very bad peaks noising the ladder
    roxy <- separate(roxy, shift=sep.index, type="pos") 
    ####################
    if(method == "cor"){
      if(length(roxy$pos) > (length(ladder)+10)){
        print(paste("WOOW too many peaks in this",who,"!! low thresholds throw too many noisy peaks, consider increasing the initial thresold for your ladder, the number of possible combinations is too high to be computed, we will have to do 10,000 samples and get the most likely sizing, you better double check this sample"))
        
        #thresh = init.thresh
        #roxy <- big.peaks.col(x, thresh)
        nono <- which(roxy$pos < avoid)
        roxy <- list(pos = roxy$pos[ -nono], hei = roxy$hei[-nono])
        roxy <- separate(roxy, shift = sep.index, type = "pos")
        pos.mod <- matrix(0, ncol=15000, nrow=length(ladder))
        for(k in 1:15000){
          pos.mod[,k] <- sort(sample(roxy$pos, size=length(ladder), replace=FALSE), decreasing=FALSE)
          
        }
        dd <- apply(pos.mod, 2, function(x5, ladder) {
          cor(x5, ladder)
        }, ladder)
        v <- which(dd == max(dd))[1]
        v2 <- which(roxy$pos %in% pos.mod[, v])
        roxy <- list(pos = pos.mod[, v], hei = roxy$hei[v2], 
                     wei = ladder)
        
      }else{
        mi=length(ladder)
        if(mi > length(roxy$pos)){
          print("ERROR!! using the initial threshold you specified we did not find enough peaks, please try reducing the 'init.thresh' argument and run the analysis again :)")
        }else{
          pos.mod <- combn(roxy$pos, m=mi)
          dd <- apply(pos.mod, 2, function(x5,ladder){cor(x5,ladder)}, ladder)
          v <- which(dd == max(dd))
          v2 <- which(roxy$pos %in% pos.mod[,v])
          roxy <- list(pos=pos.mod[,v], hei=roxy$hei[v2], wei=ladder) 
        } 
      }
    }
    ###################
    if(method == "ci"){
      z <- which(roxy$hei ==  max(roxy$hei))
      mm <- median(roxy$hei[-z]) # get the median height of the peaks
      se2.low <- (sd(roxy$hei[-z])/sqrt(length(roxy$hei[-z]))) * ci.low # produce the confidnce interval
      se2.upp <- (sd(roxy$hei[-z])/sqrt(length(roxy$hei[-z]))) * ci.upp 
      v <- which( (roxy$hei > (mm-se2.low)) & (roxy$hei < (mm+se2.upp))) # reduce the ladder to the peaks inside the 95% confidence interval
      roxy <- list(pos=roxy$pos[v], hei=roxy$hei[v])
      ## keep looking
      vv <- which(diff(roxy$pos) < dev) 
      vv2 <- vv + 1
      # start condition
      while(length(vv) > 0){
        keep <- numeric()
        for(h in 1:length(vv)){
          a1 <- vv[h]
          a2 <- vv2[h]
          a3 <- c(roxy$hei[a1],roxy$hei[a2])
          a4 <- c(a1,a2)
          keep[h] <- (a4[which(a3 == max(a3))])[1]
        }
        keep <- unique(keep)
        '%!in%' <- function(x,y)!('%in%'(x,y))
        keep2 <- unique(c(vv,vv2)[which(c(vv,vv2) %!in% keep)])
        roxy <- list(pos=roxy$pos[-keep2], hei=roxy$hei[-keep2])
        # check again
        vv <- which(diff(roxy$pos) < dev) 
        vv2 <- vv + 1
      }
      
      s1 <- length(roxy$pos)- (length(ladder) - 1)
      s2 <- length(roxy$pos)
      if(s1 <= 0){
        if(warn==TRUE){
          print("Are you sure this is a ladder channel, we did not find a clear pattern, stop a minute to check the plot")
        }
        if(draw != F){
          plot(x, ylim=c(0,mm+se2.upp), type="l")
        }
        #roxy <- list(pos=roxy$pos, hei=roxy$hei, wei= ladder)
        roxy <- list(pos=seq(1,length(ladder)) + rnorm(length(ladder),0,1), hei=seq(1,length(ladder))+ rnorm(length(ladder),0,1), wei= ladder)
      }else{roxy <- list(pos=roxy$pos[s1:s2], hei=roxy$hei[s1:s2], wei= ladder)}
    }
    ################
    # once ladder is found
    ##############################################################
    if(draw == TRUE){
      xx <- roxy$wei
      yy <- roxy$pos
      mod <- lm(yy~xx)
      xlabels <- as.vector(predict(mod, newdata=data.frame(xx=seq(0,max(ladder),by=25))))
      plot(x, type="l", main=paste("Ladder in channel provided",sep=""), xlab="", ylab="Intensity", xaxt="n")
      if(method == "cor"){
        legend("topleft", legend=paste("Correlation=",round(max(dd)), sep="")) 
      }
      axis(1, at=xlabels, labels=seq(0,max(ladder),by=25), cex.axis=0.9)
      abline(v=roxy[[1]], col="red", lty=3)
      if(method == "ci"){
        abline(h=mm, col="blue", lty=3)
        abline(h=(mm+se2.upp), col="blue", lty=3)
        abline(h=(mm-se2.low), col="blue", lty=3)
        legend("topright", legend=c("90% CI", "Peaks found"), col=c("blue", "red"), bty = "n", lty=c(3,3), cex=1)
      }else{legend("topright", legend=c("Peaks found"), col=c("red"), bty = "n", lty=c(3), cex=1)}
      text(x=roxy[[1]], rep(-200, length(ladder)), labels=ladder, cex=0.6)
      #text(x=0, mm, labels="90% CI", cex=0.6)
    }
    return(roxy)
    #########
  }

arrange.jm <- function (x, par=FALSE) 
{
  marks <- names(x)
  #marks <- gsub("S1_", "*Scaff", marks)
  dis <- dim(x)
  rows <- dis[1]
  cols <- dis[2]
  newm <- matrix(0, nrow = cols, ncol = rows + 2)
  newm[1:cols, 3:(rows + 2)] <- t(x)
  newm[which(is.na(newm))] <- "-"
  for (i in 1:cols) {
    if (length(which(newm[i, c(1:30)] %in% c("nn", "np") == TRUE)) > 0) {
      newm[i, 2] = "<nnxnp>;"
    }
    if (length(which(newm[i, c(1:30)] %in% c("lm", "ll") == TRUE)) > 0) {
      newm[i, 2] = "<lmxll>;"
    }
    if (length(which(newm[i, c(1:30)] %in% c("hh", "hk", "kk") == TRUE)) > 0) {
      newm[i, 2] = "<hkxhk>;"
    }
    if (length(which(newm[i, c(1:30)] %in% c("ab", "cd", "ac", "ad", "bc", "bd") == TRUE)) > 0) {
      newm[i, 2] = "<abxcd>;"
    }
    #if (newm[i, c(1:3)] == "-") {
    #  newm[i, 2] = "-"
    #}
  }
  newm[, 1] <- marks
  #newm[1:10,1:10]
  #v <- which(newm[, 2] == "-")
  #z <- which(newm[, 2] == 0)
  #z <- unique(c(v, w))
  if(par==F){
    newm <- newm[, -c(3, 4)]
  } else{newm <- newm}
  newm[, 3:(dim(newm)[2] - 1)] <- paste(newm[, 3:(dim(newm)[2] - 
                                                    1)], sep = "")
  rownames(newm) <- newm[, 1]
  newm3 <- as.data.frame(newm[, -1])
  namesjoin <- matrix(paste(rownames(newm3[, ]), newm3[, 1]), 
                      nrow = length(rownames(newm3)), ncol = 1)
  calls <- newm3[, 2:(dim(newm3)[2])]
  dims <- dim(calls)
  joinmap <- matrix(NA, nrow = dims[1] * 2, ncol = dims[2])
  v <- seq(1, dims[1])
  w <- seq(2, 2 * dims[1], by = 2)
  q <- seq(1, (2 * dims[1]), by = 2)
  for (i in 1:dims[1]) {
    v2 <- v[i]
    w2 <- w[i]
    q2 <- q[i]
    joinmap[w2, ] <- as.matrix(calls[v2, ])
    joinmap[q2, 1] <- as.matrix(namesjoin[v2, 1])
  }
  return(joinmap)
}

separate <-
function(g, shift=1, type="bp"){
  # shift is indicated in basepairs
  if(type == "bp"){
    vv <- which(diff(g$wei) < shift)
  }else{ vv <- which(diff(g$pos) < shift)}
  vv2 <- vv + 1
  # start condition
  while(length(vv) > 0){
    keep <- numeric()
    for(h in 1:length(vv)){
      a1 <- vv[h]
      a2 <- vv2[h]
      a3 <- c(g$hei[a1],g$hei[a2])
      a4 <- c(a1,a2)
      keep[h] <- (a4[which(a3 == max(a3))])[1]
    }
    keep <- unique(keep)
    '%!in%' <- function(x,y)!('%in%'(x,y))
    keep2 <- unique(c(vv,vv2)[which(c(vv,vv2) %!in% keep)])
    if(type=="bp"){
      g <- list(pos=g$pos[-keep2], hei=g$hei[-keep2], wei=g$wei[-keep2])  
    }else{g <- list(pos=g$pos[-keep2], hei=g$hei[-keep2])}
    
    # check again
    if(type == "bp"){
      vv <- which(diff(g$wei) < shift)
    }else{ vv <- which(diff(g$pos) < shift)}
    vv2 <- vv + 1
  }
  return(g)
}

best.layout <-
function(x){
  
  x1 <- merge(1:x,1:x) # until a graph with 9,000 spaces
  x2 <- abs(apply(x1,1,function(x,des){x <- unlist(x);y <- (x[[1]]*x[[2]])-des;return(y)},des=x))
  vw <- x1[which(x2 <2),] # which acoommodation gives the least empty space in graph
  x3 <- abs(apply(vw,1,function(x){y <- abs(x[1]-x[2]); return(y)}))
  x4 <- vw[which(x3 == min(x3))[1],] # best dimensions for the layout
  res <- as.vector(unlist(x4))
  return(res)
}

reals <-
  function(x, panel=c(100:400), shi=1, ploidy=2, left.cond=c(0.4,3), right.cond=0.2, window=0.5){
    # where x is a list with positions, heights and weights
    # panel is a vector with possible values
    # get rid of the negative base pair sizes
    neg <- which(x$wei <= 0)
    if(length(neg) >0){
      x <- list(pos=x$pos[-neg], hei=x$hei[-neg], wei=x$wei[-neg])
    }
    ##############
    picos <- numeric()
    for(d in 1:length(x$wei)){
      respi <- which(abs(panel - x$wei[d]) <= window)
      if(length(respi) > 0){
        picos[d] <- 1
      }else{picos[d] <- 0}
    }
    z1 <- which(picos == 1)
    # z1 <- which(x$wei < panel[2] & x$wei > panel[1])
    if(length(z1) >0){
      x2 <- list(pos=x$pos[z1], hei=x$hei[z1], wei=x$wei[z1]) 
      ###############################################
      # if peaks are closer than 1 bp just select the tallest one and get rid of the other one
      x3 <- separate(x2, shi, type="bp")
      ################################################
      ### eliminate peaks that are not at least half the size of the tallest peak
      # but this should only happen for peaks before the tallest peak
      highest <- which(x3$hei == max(x3$hei))
      che <- x3$hei[1:highest[1]]
      ha <- which(che >= (x3$hei[highest] * left.cond[1]))
      cha <- x3$wei[1:highest[1]] 
      ha2 <- unique(c(which(abs(cha - x3$wei[highest]) >= left.cond[2]), highest)) # are further than, greater than
      # rigth side condition
      chu <- x3$hei[highest[1]:length(x3$hei)]
      hu <- (highest[1]:length(x3$hei))[which(chu > (max(x3$hei) * right.cond))]
      # intersection
      ss1 <- intersect(ha, ha2)
      ha3 <- unique(c(ss1, hu))
      if(length(ha3) > 0){
        x3 <- list(pos = x3$pos[ha3], hei = x3$hei[ha3], wei = x3$wei[ha3])
      }else{x3 <- x3}
      ### now lets select the peaks that are only after the tallest
      #ho <- which(x3$hei > (max(x3$hei)*right.cond))
      #if(length(ho) > 0){
      #  x3 <- list(pos = x3$pos[ho], hei = round(x3$hei[ho]), wei = x3$wei[ho])
      #}else{x3 <- x3}
      ########
      #he <- which(x3$hei > (max(x3$hei)*whenheight))
      #x3 <- list(pos = x3$pos[he], hei = round(x3$hei[he]), 
      #          wei = x3$wei[he])
      
      ##########################################################################
      ### BASED ON PLOIDY SELECT HOW MANY PEAKS YOU WANT
      ###############################################
      # if more than 2 peaks select the bigger peaks
      z2 <- length(x3$pos)
      ## one peak find, then duplicate
      if(z2 == 1){
        x4 <- list(pos=rep(x3$pos,ploidy), hei=rep(x3$hei,ploidy), wei=rep(x3$wei,ploidy)) 
      }
      # more than one peak found, we need to find 
      if(z2 > 1){
        # find all different combinations possible
        toget <- sort(x3$hei, decreasing=T)[1:ploidy]
        z3 <- which(x3$hei %in% toget)
        x4 <- list(pos=x3$pos[z3], hei=round(x3$hei[z3]), wei=x3$wei[z3]) 
      }#else{x4 <- x3}
    } else{x4 <- list(pos=rep(0, ploidy), hei=rep(0, ploidy), wei=rep(0,ploidy)) }
    ###############################################
    return(x4)
    
  }

ladder.info.attach <- function(stored, ladder, 
                               channel.ladder=NULL, method="iter2", 
                               ladd.init.thresh=NULL, env = parent.frame(), 
                               prog=TRUE, draw=TRUE, attempt=10){
  all.names <- names(stored)
  
  dev=50; warn= FALSE
  if(is.null(channel.ladder)){
    channel.ladder <- dim(stored[[1]])[2]
  }else{channel.ladder <- channel.ladder}
  layout(matrix(1:3,nrow=3, ncol=1))
  #####################
  ### this part extracts all the models for each single plant in my plants
  list.ladders <- lapply(stored, function(x){y <- x[,channel.ladder]; return(y)})
  # extract ladder channels for all plants
  #to.add <- names(stored)
  #for(f in 1:length(list.ladders)){
  #  attributes(list.ladders[[f]])$name <- to.add[f]
  #}
  ##
  for(t in 1:length(list.ladders)){
    attributes(list.ladders[[t]]) <- list(mycomm=names(list.ladders)[t])
  }
  
  if(prog==TRUE){
    res <- lapply_pb(list.ladders, find.ladder, init.thresh=ladd.init.thresh, ladder=ladder, dev=dev, warn=warn, method=method, draw=draw, attempt=attempt)
  }
  if(prog==FALSE){
    res <- lapply(list.ladders, find.ladder, ladder=ladder, dev=dev, warn=warn,  method=method,init.thresh=ladd.init.thresh, draw=draw, attempt=attempt)
  }
  env$list.data.covarrubias <- res
  
  cat("\nSizing process complete. Information has been stored in the environment for posterior functions.\nFor example to be used by the overview2() or score.markers() functions.")
  
  correlations <- unlist(lapply(res, function(x){x$corr}))
  if(length(which(correlations < .92)) >0){
    cat(paste("\nWe did not find a good ladder in",length(which(correlations < .92)),"sample(s). \nIf you wish to correct it you can try one of the following:\n"))
    cat("\na) The value of ladd.init.thresh might be too low, making noisy peaks too be abundant \n     Solution-- make sure your initial value 'init.thresh' is not below 200 RFUs\nb) You can continue your analysis without worrying for those samples or removing. Identify them as:\n     corro <- unlist(lapply(list.data.covarrubias, function(x){x$corr}))\n     (bad <- which(corro < .9999))\nc) MOST IMPORTANT! you can correct manually the bad samples using the 'ladder.corrector()' function providing the names of the bad samples (below), your ladder, and the information from the 'storing.inds' function, type ?ladder.corrector\n\nNames of the bad sample(s):\n")
    # a) The first peaks of your ladder are in a very noisy area \n     Solution-- try discarding some initial values of your ladder, start one by one \n
    #print(all.names[which(correlations < .92)])
   # if(length(which(correlations < .92)) > (length(correlations)*.05)){
    #  ww <- which(correlations < .92)[1]
    #  prov <- stored[[ww]][,dim(stored[[ww]])[2]]
    #  mind <- big.peaks.col(prov, ladd.init.thresh)
    #  layout(matrix(1,1,1))
    #  plot(prov, type="l", ylim=c(0,4000))
    #  abline(v=mind$pos, lty=3, col="red")
      #legend("topright", legend=paste(length(mind$pos),"peaks found,",length(ladder),"ladder peaks expected"), bty="n")
    #  legend("center", legend=c(paste(ladder[1:round(length(ladder)/2)],"",sep="", collapse = "  ")),bty="n", cex=.7)
    #  legend("topleft", legend="?",bty="n", cex=2)
    #  cat(paste("We will show you a plot of the ladder channel of one bad sample, you provided a ladder with",length(ladder), "expected peaks and there's",length(mind$pos), "peaks in the channel, make sure that 1st peak of your ladder is not in an area with too much noise \nSolution-- Consider discarding some initial values of your ladder \n "))
      
    #}
  }
  ######################
  layout(matrix(1,1,1))
  
  bads <- all.names[which(correlations < .92)]
  if(length(bads)>0){
    return(bads) 
  }
}



##################################################################################################
#Startup function
#this function is executed once the library is loaded
.onAttach = function(library, pkg)
{
  Rv = R.Version()
  if(!exists("getRversion", baseenv()) || (getRversion() < "2.1"))
    stop("This package requires R 2.1 or later")
  assign(".Fragman.home", file.path(library, pkg),
         pos=match("package:Fragman", search()))
  Fragman.version = "1.0.9 (2018-02-01)"
  assign(".Fragman.version", Fragman.version, pos=match("package:Fragman", search()))
  if(interactive())
  {
    packageStartupMessage(paste("## ========================================================= ## "),appendLF=TRUE)
    packageStartupMessage(paste("## ========================================================= ## "),appendLF=TRUE)
    packageStartupMessage(paste("# Fragman: An R package for Fragment Analysis ", Fragman.version, ". ",sep=""),appendLF=TRUE)
    packageStartupMessage("# Author: Covarrubias-Pazaran et al.",appendLF=TRUE)
    packageStartupMessage("# Published: BMC Genetics 17(62):1-8",appendLF=TRUE)
    packageStartupMessage("# Supported and partially funded by:", appendLF=TRUE)
    packageStartupMessage("#    + Council of Science and Technology (CONACYT)", appendLF=TRUE)
    packageStartupMessage("#    + US Department of Agriculture (USDA-ARS)", appendLF=TRUE)
    packageStartupMessage("# Type 'help(Fragman)' for summary information",appendLF=TRUE)
    packageStartupMessage("# Type 'citation(Fragman)' to know how to cite this package",appendLF=TRUE)
    packageStartupMessage(paste("## ========================================================= ## "),appendLF=TRUE)
    packageStartupMessage(paste("## ========================================================= ## "),appendLF=TRUE)
  }
  invisible()
}

homogenize.to.parentals <-
function(x, parents, window){
  #d1 <- data.frame(matrix(parents, nrow=1))
  
  alls <- numeric()
  for(i in 1:length(x$wei)){
    bb <- abs(x$wei[i] - parents)
    bb2 <- which(bb < window)[1]
    if(is.na(bb2)){
      alls[i] <- x$wei[1]
    }else{alls[i] <- parents[[bb2]]}
    
  }
  x$wei <- alls
  # returns alleles but in parental conformation
  return(x)
}

pullup <- function(mati, plotting=FALSE, channel=4){
  
  ### first pull up negative peaks
  #for(f in 1:(dim(mati)[2] -1)){
  #s1 <- (mati[,f]* -1)
  #starting <- median(mati[,f])
  #mati[which(s1 > (starting+200))] <- abs(mati[which(s1 > (starting+200))]) * 2
  #}
  ### now the real pullup
  all.cols <- apply(mati[,-channel],1,sum)
  #plot(all.cols, type="l", xlim=c(0,5000))
  #head(mati)
  rows <- dim(all.cols)[1]
  peak.all.cols <- big.peaks.col(all.cols, tre=300)
  #### which are the regions of the peaks to apply pull up, set difference to 100
  vv1 <- which(diff(peak.all.cols$pos) > 100); vv1 <- c(vv1, length(peak.all.cols$pos)) # where peaks cut
  vv2 <- vv1+1; vv2 <- vv2[-length(vv2)]; vv2 <- c(1,vv2)
  regions <- apply(data.frame(cbind(vv2,vv1)), 1, function(x,pos){y <- pos[x[1]:x[2]]; return(y)}, peak.all.cols$pos)
  #########################
  #if(!is.na(regions)){
    for(i in 1:length(regions)){
      # find which is the maximum for that region in all the channels
      maxis <- apply(mati[,-channel], 2, function(x, regi){max(x[regi])}, regi=regions[[i]])
      ch <- c(1:length(maxis))
      wiii <- which(maxis == max(maxis)) # channel where this peak is real
      rest <- ch[-wiii] # channels where the peak is not real
      # adjust by decreasing 25% the intensity of the real channel
      decrease <- regions[[i]][1]:(regions[[i]][length(regions[[i]])])
      mati[decrease,rest] <- mati[decrease,rest] - (mati[decrease,wiii]*.3)
    }
  #}
  
  #all.inds.mats2 <- apply(mati, 2, function(x){x[which(x < 0)] <- 0; return(x)}) 
  all.inds.mats2 <- apply(mati, 2, function(x){x[which(x < 0)] <- x[which(x < 0)]; return(x)}) # THIS PART SEEMS TO BE IMPORTANT FOR DONT GETTING DOUBLE LINES
  #all.inds.mats2  <- data.frame(all.inds.mats2)
  if(plotting == TRUE){
    layout(matrix(1:2,2,1))
    plot(mati[,1], type="l", col="blue")
    colorsh <- c("cornflowerblue", "chartreuse4", "gold2", "red", "orange", "purple")
    for(i in 2:dim(mati)[2]){
      lines(mati[,i], col=colorsh[i])  
    }
    plot(all.inds.mats2[,1], type="l", col="blue")
    colorsh <- c("cornflowerblue", "chartreuse4", "gold2", "red", "orange", "purple")
    for(i in 2:dim(all.inds.mats2)[2]){
      lines(all.inds.mats2[,i], col=colorsh[i])  
    }
  }
  layout(matrix(1,1,1))
  
  return(all.inds.mats2)
}

transfft <- function(sn, top=0.3){
  sn.fft = fft(sn)
  #plot(Re(sn.fft), type = "l")
  #qq <- round(length(sn)*.08)
  qq <- length(sn)*top # 3000
  #plot(Re(sn.fft), xlim = c(0, 1000), type = "l")
  sn.fft[qq:(length(sn)-qq)] = 0 + 0i
  sn.ifft = fft(sn.fft, inverse = TRUE)/length(sn.fft)
  return(Re(sn.ifft))
} 

ladder.corrector <- function(stored, to.correct, ladder, thresh=200, 
                             env = parent.frame(), ...){
  list.data <- env$list.data.covarrubias
  
  vvv <- which(names(list.data) %in% to.correct)
  chan <- dim(stored[[1]])[2]
  ###
  #--
  ###
  for(j in 1:length(vvv)){
    cat(paste("\nSTARTING SAMPLE",j,""))
    www <- vvv[j]
    x <- stored[[www]][,chan]
    roxy <- big.peaks.col(x, thresh)
    
    outlier_values <- boxplot.stats(roxy$hei)$out
    if(length(outlier_values)>0){
      ylimi <- max(sort(roxy$hei[which(roxy$hei != outlier_values)], decreasing=TRUE)) 
    }else{
      ylimi <- max(sort(roxy$hei, decreasing=TRUE)) 
    }
        
    layout(matrix(1,1,1))
    plot(x, type="l", ylim=c(0,ylimi), main=paste("Provitional plot for\n",to.correct[j]), las=2, cex.main=.8, ...);grid()
    abline(h=c(20,50,100,200), col="red",lty=3)
    axis(2, at=c(20,50,100,200),labels = c(20,50,100,200), las=1, col.ticks ="red")
    cat("\nPlease specify if you prefer manual or automatic scoring. \nType the character 'm' or 'a' (autom is preferred):\n")
    type <- readline()
    type <- gsub("\'","",type) # in case the user gave character
    type <- gsub("\"","",type)
    
    basicc <- c("a","m")
    choco <- length(intersect(basicc,type))
    while((choco)==0){
      cat("\nType the character 'm' or 'a':\n")
      type <- readline()
      type <- gsub("\'","",type) # in case the user gave character
      type <- gsub("\"","",type)
      choco <- length(intersect(basicc,type))
    }
    
    if(type=="a"){
      cat("Please provide the threshold value for the ladder in RFUs (number):")
      hisladder <- as.numeric(readline())
      cat("Should we reduce the search in the x-axis to certain peaks?.If none type the 'Esc' key\notherwise click over the plot where where the start is (one click) and \nthe end (another click), and finally type the 'Esc' key")
      #tolook <- as.numeric(readline())
      mine <- locator(type="p", pch=20, col="red")$x
      if(is.null(mine)){
        reducing <- NULL
      }else{
        tolook <- round(min(mine)):round(max(mine))
        reducing <- tolook
      }
      
      provi <- find.ladder(x, ladder=ladder,init.thresh = hisladder, 
                           draw = FALSE, method="iter2", reducing = reducing)
      abline(v=provi$pos, col="red",lty=3)
      list.data[[www]]$pos <- provi$pos
      list.data[[www]]$hei <- provi$hei
      list.data[[www]]$wei <- provi$wei
      list.data[[www]]$corr <- provi$corr
      cat("\nSample adjusted.")
    }else if(type=="m"){
      cat("Please provide the threshold value for the ladder in RFUs (number):")
      hisladder <- as.numeric(readline())
      x <- stored[[www]][,chan]
      roxy <- big.peaks.col(x, hisladder)
      cat(paste("\nYou need you to click over peaks (",length(ladder),") corresponding to the following ladder: \n\n", paste(ladder, collapse = " "), "\n\nPress 'esc' when you are done. \n"))
      good <- locator(type="p", pch=20, col="red")$x
      #############################################
      ## now use such values to correct the model
      corrected <- numeric()
      for(k in 1:length(good)){
        yoyo <- abs(roxy$pos - good[k])
        corrected[k] <- roxy$pos[which(yoyo == min(yoyo))]
      }
      corrected.hei <- roxy$hei[which(roxy$pos %in% corrected)]
      if(length(corrected) != length(ladder)){
        cat("You did not select the same number of peaks than the number of elements \nof your ladder. Getting more likely peaks. Please check the resulting peaks \nand repeat if needed. ")
      }else{
        list.data[[www]]$pos <- corrected
        list.data[[www]]$hei <- corrected.hei
        list.data[[www]]$wei <- ladder
        list.data[[www]]$corr <- cor(ladder, corrected)
      }
      ##############################################
    }
    ###
  }
  ###
  #--
  ###
  layout(matrix(1:3,nrow = 3,ncol=1))
  for(l in 1:length(vvv)){
    www <- vvv[l]
    x <- stored[[www]][,chan]
    limi <- sort(list.data[[www]]$hei, decreasing = TRUE)
    plot(x, type="l", xaxt="n", ylim=c(0,(limi[3]+100)), cex.axis=0.6, las=2,  col=transp("grey35",0.7), ylab="RFU", xlab="", lwd=2, cex.main=.8,main=paste("Adjusted ladder for\n",to.correct[l]))#xlim=c((min(list.data[[www]]$pos)-100),(max(list.data[[www]]$pos)+100)),
    axis(1, at=list.data[[www]]$pos, labels=list.data[[www]]$wei, cex.axis=0.6)
    points(x=list.data[[www]]$pos, y=list.data[[www]]$hei,cex=1.1, col=transp("black",0.85))
    points(x=list.data[[www]]$pos, y=list.data[[www]]$hei, pch=20, col=transp("red",0.7))
    legend("topleft", legend=paste("Correlation:",round(list.data[[www]]$corr, digits=4), sep=""), bty="n")
    legend("topright", legend=c("Peaks selected"), col=c("red"), bty = "n", pch=c(20), cex=0.85)
    
  }
  env$list.data.covarrubias <- list.data
  layout(matrix(1,1,1))
  cat("\nJOB DONE!!! ladder has been adjusted for the samples provided")
}

find.ladder <-function(x, ladder, draw=TRUE, dev=50, warn=TRUE, init.thresh=NULL, sep.index=8, method=NULL, reducing=NULL, who="sample", attempt=10, cex.title=0.8){
  #roundUP <- function(x) 10^ceiling(log10(x))
  
  hohoho <- big.peaks.col(x[1:length(x)], 100)
  
  # plot(x, type="l", ylim = c(0,2000))
  # abline(v=hohoho$pos, lty=3, col="red")
  
  if(is.null(method)){
    ## check if the ladder passes the length of x of is a small ladder
    tototo <- big.peaks.col(x[1:length(x)], median(hohoho$hei)*2)
    close <- (length(x) - tototo$pos[length(tototo$hei)])
    if(close < 100){
      method="iter"
    }else{
      method="iter2"
    }
  }
  
  if(is.null(init.thresh)){
    if(mean(x) > 1000){
      #hohoho2 <- big.peaks.col(x[1:length(x)], 2*median(hohoho$hei))
      init.thresh <- quantile(hohoho$hei,.95)
    }else{
      init.thresh <- median(hohoho$hei)/2.2#quantile(hohoho$hei,.32)
    }
  }
  
  MSE <- function(x,y){
    X <- cbind(1, x)
    qr.X <- qr(X)
    b <- t(qr.Q(qr.X)) %*% y
    R <- qr.R(qr.X)
    beta <- as.vector(backsolve(R, b))
    fit <- X%*%beta
    res <- list(mse=sum((y-fit)^2), beta=beta[2])
    return(res)
  }
  MSE2 <- function(x,y){
    X <- cbind(1, x)
    qr.X <- qr(X)
    b <- t(qr.Q(qr.X)) %*% y
    R <- qr.R(qr.X)
    beta <- as.vector(backsolve(R, b))
    fit <- X%*%beta
    mse <- sum((y-fit)^2)# is actually SSE
    sst <- sum((y-mean(y))^2)
    r2 <- 1 - (mse/sst) #cor(x,y)^2#mse/sum((y-mean(y,na.rm=TRUE))^2)
    res <- list(mse=mse, beta=beta, r2=r2)
    return(res)
  }
  MSE3 <- function(mix,miy){
    X <- cbind(1, mix)
    qr.X <- qr(X)
    b <- t(qr.Q(qr.X)) %*% miy
    R <- qr.R(qr.X)
    beta <- as.vector(backsolve(R, b))
    fit <- X%*%beta
    mse <- sum((miy-fit)^2)# is actually SSE
    sst <- sum((miy-mean(miy))^2)
    r2 <- 1 - (mse/sst) #cor(x,y)^2#mse/sum((y-mean(y,na.rm=TRUE))^2)
    res <- list(mse=mse, beta=beta, r2=r2)
    return(res)
  }
  ##### stablish a minimum threshold
  thresh=init.thresh
  roxy <- big.peaks.col(x[1:length(x)], thresh) # find all peaks with that threshold
  
  if(!is.null(reducing)){
    nono <- which(roxy$pos %in% reducing)
    roxy <- lapply(roxy, function(x){x[nono]})
    #roxy <- list(pos = roxy$pos[ -nono], hei = roxy$hei[-nono])
  }
  
  nnn <- length(roxy$pos)
  
  # plot(x, type="l", ylim = c(0,2000))
  # abline(v=roxy$pos, lty=3, col="red")
  
  fdsa=1
  while(nnn < length(ladder)){
    #print(paste("Warning: Your ladder for the sample",attr(x,"name") ,"is bad, very low intensity in RFU was found, we are reducing the threshold 2x trying to find it, don't worry too much the analysis will keep going anyways"))
    
    if(fdsa == 1){
      cat("\nReducing threshold 2x to find ladder \n")
      #cat("There's too many ladder peaks expected and not many peaks found in this sample\nPlease make sure all your ladder peaks specified show up in the sample\nOtherwise provide a smaller ladder so all peaks can be found\n")
    }
    thresh=thresh/2
    roxy <- big.peaks.col(x[1:length(x)], thresh) # find all peaks with that threshold
    nnn <- length(roxy$pos)
    fdsa<-fdsa+1
  }
  ####
  #use the indicator peak at the beggining which is usually the tallest peak in the ladder
  # get rid of very bad peaks noising the ladder
  whot <- length(roxy$pos)*.2
  what <- which(roxy$hei == max(roxy$hei))
  #what <- which(roxy$hei == sort(roxy$hei, decreasing = TRUE)[2])
  # if you tallest peak is far beyond don't adjust
  #if(what > whot){
  #  roxy <- roxy
  #}else{ max(roxy$hei)
  roxy <- separate(roxy, shift=sep.index, type="pos")
  
  
  #} plot(x, type="l",xlim=c(2000,7000)); abline(v=roxy$pos, col="red", lty=3)
  #####################
  if (method == "iter") {
    ii <- which(roxy$hei == max(roxy$hei)) + 1
    iii <- length(roxy$hei)
    roxy <- list(pos = roxy$pos[ii:iii], hei = roxy$hei[ii:iii])
    
    step1 <- combn(roxy$pos[1:attempt], 3)
    step2 <- apply(step1/10, 2, MSE, y = ladder[1:3])
    mse <- unlist(lapply(step2, function(x) {
      x$mse
    }))
    covs <- apply(step1, 2, function(x, y) {
      cov(x, y)
    }, y = ladder[1:3])
    step2 <- mse * covs
    step3 <- step1[, which(step2 < sort(step2, decreasing = FALSE)[20])]
    step4 <- apply(step3, 2, function(x, y) {
      which(y %in% x)
    }, y = roxy$pos)
    caller <- function(roxy, www, ladder.call, x) {
      threshold <- length(x)
      posi <- numeric()
      fact2 <- length(ladder.call)
      expect <- roxy$pos[www]
      xxx <- ladder.call[c(1:3)]
      modx <- lm(expect ~ poly(xxx, degree = 1))
      expecto <- predict(modx, data.frame(xxx = ladder.call))
      ladder.call <- ladder.call[which(expecto < threshold * 
                                         0.85)]
      available <- length(roxy$pos) - length(ladder.call)
      ava2 <- length(ladder.call) - abs(available)
      if (available > 0) {
        if ((length(ladder.call) - 1) < 3) {
          tope <- length(ladder.call)
        }
        else {
          tope <- length(ladder.call) - 1
        }
      }
      else {
        tope <- ava2 - 2
      }
      expect <- rep(NA, tope + 1)
      for (i in 3:tope) {
        if (i == 3 & i != tope) {
          expect[1:3] <- roxy$pos[www]
          xxx <- ladder.call[c(1:3)]
          mod <- MSE2(xxx, expect[1:3])
          beta <- (mod)$beta
          expecto <- as.vector(beta[1] + matrix(ladder.call) %*% 
                                 beta[-1])
          act <- roxy$pos[-which(roxy$pos %in% expect)]
          yoyo <- abs(expecto[i + 1] - act)
          good <- which(yoyo == min(yoyo,na.rm = TRUE))
          expect[i + 1] <- act[good]
          if (mod$r2 < 0.9) {
            i = tope
          }
        }
        if (i > 3 & i <= 5) {
          xx <- ladder.call[c(1:i)]
          mod <- MSE2(xx, expect[1:i])
          beta <- (mod)$beta
          expecto <- as.vector(beta[1] + matrix(ladder.call) %*% 
                                 beta[-1])
          act <- roxy$pos[-which(roxy$pos %in% expect)]
          yoyo <- abs(expecto[i + 1] - act)
          #print(min(yoyo))
          good <- which(yoyo == min(yoyo, na.rm = TRUE))
          expect[i + 1] <- act[good]
          if (mod$r2 < 0.9) {
            i = tope
          }
        }
        if (i > 5) {
          xx <- cbind(ladder.call[c(1:i)], ladder.call[c(1:i)]^2, 
                      ladder.call[c(1:i)]^3, ladder.call[c(1:i)]^4)
          mod <- MSE2(xx, expect[1:i])
          beta <- (mod)$beta
          if (length(which(is.na(beta))) > 0) {
            beta[which(is.na(beta))] <- 0
          }
          toto <- cbind(matrix(ladder.call), matrix(ladder.call)^2, 
                        matrix(ladder.call)^3, matrix(ladder.call)^4)
          expecto <- cbind(rep(1, dim(toto)[1]), toto) %*% 
            beta
          act <- roxy$pos[-which(roxy$pos %in% expect)]
          yoyo <- abs(expecto[i + 1] - act)
          good <- which(yoyo == min(yoyo,na.rm = TRUE))
          expect[i + 1] <- act[good]
          if (is.na(mod$r2)) {
            mod$r2 <- 0.1
          }
          if (mod$r2 < 0.9) {
            i = tope
          }
        }
        if (i == tope & i != 3) {
          if (i < 5) {
            expect[1:3] <- roxy$pos[www]
            xx <- ladder.call[c(1:i)]
          }
          else {
            xx <- cbind(ladder.call[c(1:i)], ladder.call[c(1:i)]^2, 
                        ladder.call[c(1:i)]^3, ladder.call[c(1:i)]^4)
          }
          mod <- MSE2(xx, expect[1:i])
          beta <- (mod)$beta
          if (length(which(is.na(beta))) > 0) {
            beta[which(is.na(beta))] <- 0
          }
          if (i < 5) {
            toto <- cbind(matrix(ladder.call))
          }
          else {
            toto <- cbind(matrix(ladder.call), matrix(ladder.call)^2, 
                          matrix(ladder.call)^3, matrix(ladder.call)^4)
          }
          expecto <- cbind(rep(1, dim(toto)[1]), toto) %*% 
            beta
          act <- roxy$pos[-which(roxy$pos %in% expect)]
          yoyo <- abs(expecto[i + 1] - act)
          good <- which(yoyo == min(yoyo, na.rm = TRUE))
          expect[i + 1] <- act[good]
        }
        if (i == tope & i == 3) {
          expect[1:3] <- roxy$pos[www]
        }
      }
      posi <- expect
      tutu <- abs(length(x) - posi)
      posi <- posi[1:which(tutu == min(tutu, na.rm = TRUE))]
      heii <- roxy$hei[which(roxy$pos %in% posi)]
      fact3 <- length(posi)/fact2
      if (length((posi)) < 6) {
        fact <- summary(lm(ladder.call[1:length(posi)] ~ 
                             poly(posi, degree = length((posi)) - 1)))$r.squared * 
          fact3
      }
      else {
        fact <- summary(lm(ladder.call[1:length(posi)] ~ 
                             poly(posi, degree = 5)))$r.squared * fact3
      }
      roxy2 <- list(pos = posi, hei = heii, wei = ladder.call[1:length(posi)], 
                    corr = abs(cor(ladder.call[1:length(posi)], posi)), 
                    error = fact)
      return(roxy2)
    }
    rt <- apply(data.frame(step4), 2, FUN = caller, roxy = roxy, 
                ladder.call = ladder, x = x)
    corrs3 <- unlist(lapply(rt, function(x) {
      x$error
    }))
    roxy3 <- rt[[which(corrs3 == max(corrs3))]]
    if (draw == TRUE) {
      limi <- sort(roxy3$hei, decreasing = TRUE)
      plot(x, type = "l", xaxt = "n", ylim = c(0, (limi[3] + 
                                                     1000)), cex.axis = 0.6, las = 2, xlim = c((min(roxy3$pos) - 
                                                                                                  100), (max(roxy3$pos) + 100)), col = transp("grey35", 
                                                                                                                                              0.7), ylab = "RFU", xlab = "", lwd = 2, main = attributes(x)$mycomm, 
           cex.main = cex.title)
      axis(1, at = roxy3$pos, labels = roxy3$wei, cex.axis = 0.6)
      points(x = roxy3$pos, y = roxy3$hei, cex = 1.1, col = transp("black", 
                                                                   0.85))
      points(x = roxy3$pos, y = roxy3$hei, pch = 20, col = transp("red", 
                                                                  0.7))
      legend("topleft", legend = paste("Correlation:", 
                                       round(roxy3$corr, digits = 4), sep = ""), bty = "n")
      legend("topright", legend = c("Peaks selected"), 
             col = c("red"), bty = "n", pch = c(20), cex = 0.85)
    }
    roxy <- roxy3
  }
  if(method == "iter2"){
    #caller
    last <- length(roxy$pos)
    lld <- length(ladder)
    if((last-attempt) < 0){
      #roxy2 <- roxy#list(pos=posi, hei=heii, wei=ladder.call[1:length(posi)], corr=abs(cor(ladder.call[1:length(posi)],posi)), error=fact)#sum(error, na.rm=TRUE))
      roxy$wei <- ladder[1:length(roxy$pos)]
      roxy$corr <- 0
      roxy$error <- 0
      if(draw == TRUE){
        limi <- sort(roxy$hei, decreasing = TRUE)
        plot(x, type="l", xaxt="n", ylim=c(0,(limi[3]+1000)), cex.axis=0.6, las=2,  col=transp("grey35",0.7), ylab="RFU", xlab="", lwd=2, main=attributes(x)$mycomm, cex.main=cex.title)
        axis(1, at=roxy$pos, labels=roxy$wei, cex.axis=0.6)
        points(x=roxy$pos, y=roxy$hei,cex=1.1, col=transp("black",0.85))
        points(x=roxy$pos, y=roxy$hei, pch=20, col=transp("red",0.7))
        legend("topleft", legend=paste("Correlation:",round(roxy$corr, digits=4), sep=""), bty="n")
        legend("topright", legend=c("Peaks selected"), col=c("red"), bty = "n", pch=c(20), cex=0.85)
        legend("center", legend=c("Intensity too low to be detected"), col=c("red"), bty = "n", cex=0.85)
      }
    }else{
    # by default attempt=10 making all combinations of 1st 10 peaks
    step1 <- combn(roxy$pos[last:(last-attempt)],3)
    #### check MSE
    step2 <- apply(step1/10, 2, MSE, y=ladder[lld:(lld-2)]) # function(x,y){sum((summary(lm(I(x)~y))$residuals)^2)}create a custom function with matrices to extract residuals and calculate MSE
    #### mean squared errors
    mse <- unlist(lapply(step2, function(x){x$mse}))
    ##### covariances
    covs <- apply(step1, 2, function(x,y){cov(x,y)}, y=ladder[lld:(lld-2)])
    ##### scores for selection
    step2 <- mse #* covs 
    ##### selected based on scores
    step3 <- step1[,which(step2 < sort(step2, decreasing=FALSE)[20])] # 15 models with least MSE
    # winner = 1
    # for(i in 1:dim(step3)[2]){ # see selected
    #   plot(x,type="l", main=i)
    #   abline(v=step3[,i], lty=3, col="red")
    # }
    # position of peaks-combinations selected 
    step4 <- apply(step3,2,function(x,y){sort(which(y %in% x), decreasing = TRUE)}, y=roxy$pos) # which peaks are
    #caller  6
    ############
    #-----------
    ############
    caller <- function(roxy,www, ladder.call,x){
      threshold <- length(x)
      
      last3 <- length(ladder.call):(length(ladder.call)-2)
      posi <- numeric()
      fact2 <- length(ladder.call)
      # www <- step4[,5]; ladder.call=ladder
      ############################################################
      ## short initial model to avoid people using a ladder.call too long, we cut the ladder.call if necessary
      expect <- roxy$pos[www] # initial positions for this combo (last)
      xxx <- ladder.call[last3] # ladder positions assumed (last)
      modx <- lm(expect~poly(xxx, degree=1))
      ## assuming model is correct predict where should be the rest of the ladder
      expecto <- predict(modx, data.frame(xxx=ladder.call))# + facto
      #ladder.call <- ladder.call[which(expecto < threshold*.85)]
      
      available <- length(roxy$pos) - length(ladder.call) # there's x extra peaks
      ava2 <-  length(ladder.call) - abs(available) # is it more than the user specified??
      if(available < 0){ # if there is less peaks available than existing in ladder BAD USER
        #if((length(ladder.call)-1) < 3){
        #  tope <- length(ladder.call)
        #}else{tope <- length(ladder.call)-1}
        tope <- 3 # is bad sample
        
      }else{
        tope <- length(ladder.call)
      } # if there is more peaks available than expecting in ladder GOOD USER
      
      #if(available > 0){tope <- length(ladder.call)-1}
      #}else{tope <- abs(available)-1}
      #####
      #tope <- length(ladder.call) #- 3
      expect <- rep(NA,tope)# (length(ladder.call)-maxo)
      
      lenlad <- length(ladder.call)
      
      for(i in 3:(tope-1)){ # for all possible peaks
        if(i == 3 & i != tope){# initial step
          
          expect[tope:(tope-2)] <- roxy$pos[www]
          xxx <- ladder.call[last3]
          #mod1 <- lm(expect[1:3]~poly(xxx, degree=1))
          #expecto <- predict(mod1, data.frame(xxx=ladder.call))# + facto
          mod <- MSE3(mix=xxx, miy=expect[tope:(tope-2)])
          beta <- (mod)$beta # extract intercept and slope
          ## predict the rest of the ladder
          expecto <- as.vector(beta[1] + matrix(sort(ladder.call, decreasing = TRUE)) %*% beta[-1])
          # remove missing data
          condo <- sort(expect[which(!is.na(expect))], decreasing = TRUE)
          # keep roxy values that were picked already
          act <- sort(roxy$pos[-which(roxy$pos %in% condo)], decreasing = TRUE)
          # see which is the next most likely value
          yoyo <- abs(expecto[i+1] - act)
          good <- which(yoyo == min(yoyo, na.rm = TRUE))
          
          qwer <- i
          qwer2 <- length(expect)-qwer
          expect[qwer2] <- act[good]
          # --------------------------------
          #if(summary(mod1)$r.squared < .9){
          if(mod$r2 < .9){
            i= tope #length(ladder.call) - maxo
          }
          # --------------------------------
        }else if(i > 3 & i <= 5 ){
          
          xx <- ladder.call[c(lenlad:(lenlad-(i-1)))]
          #mod1 <- lm(expect[1:i]~poly(xx, degree=1))
          #expecto <- predict(mod1, data.frame(xx=ladder.call))
          
          mod <- MSE3(mix=xx, miy=expect[tope:qwer2])
          beta <- (mod)$beta # extract intercept and slope
          ## predict the rest of the ladder
          expecto <- as.vector(beta[1] + matrix(sort(ladder.call, decreasing = TRUE)) %*% beta[-1])
          # remove missing data
          condo <- sort(expect[which(!is.na(expect))], decreasing = TRUE)
          # keep roxy values that were picked already
          act <- sort(roxy$pos[-which(roxy$pos %in% condo)], decreasing = TRUE)
          # see which is the next most likely value
          yoyo <- abs(expecto[i+1] - act)
          good <- which(yoyo == min(yoyo, na.rm = TRUE))
          qwer <- i
          qwer2 <- length(expect)-qwer
          expect[qwer2] <- act[good]
          # --------------------------------
          #if(summary(mod1)$r.squared < .9){
          if(mod$r2 < .9){
            i= tope #length(ladder.call) - maxo
          }
          # --------------------------------
        }else if(i > 5 ){
          #xx <- ladder.call[c(1:i)]
          #mod1 <- lm(expect[1:i]~poly(xx, degree=4, raw=TRUE))
          #expecto <- predict(mod1, data.frame(xx=ladder.call))
          ladder.call2 <- sort(ladder.call, decreasing = TRUE)
          expect2 <- sort(expect, decreasing = TRUE)
          xx <- cbind(ladder.call2[c(1:i)])#,ladder.call2[c(1:i)]^2,ladder.call2[c(1:i)]^3,ladder.call2[c(1:i)]^4)
          mod <- MSE3(mix=xx, miy=expect2)
          beta <- (mod)$beta
          if(length(which(is.na(beta))) > 0){
            beta[which(is.na(beta))] <- 0
          }
          toto <- cbind(matrix(ladder.call2))#,matrix(ladder.call2)^2, matrix(ladder.call2)^3,matrix(ladder.call2)^4)
          expecto <- cbind(rep(1,dim(toto)[1]),toto) %*% beta
          
          # remove missing data
          condo <- sort(expect[which(!is.na(expect))], decreasing = TRUE)
          # keep roxy values that were picked already
          act <- sort(roxy$pos[-which(roxy$pos %in% condo)], decreasing = TRUE)
          # see which is the next most likely value
          yoyo <- abs(expecto[i+1] - act)
          good <- which(yoyo == min(yoyo, na.rm = TRUE))
          qwer <- i
          qwer2 <- length(expect)-qwer
          expect[qwer2] <- act[good]
          # --------------------------------
          #if(summary(mod1)$r.squared < .9){
          if(is.na(mod$r2)){ # if model is too bad just assign a zero value
            mod$r2 <- 0.1
          }
          if(mod$r2 < .9){
            i= tope #length(ladder.call) - maxo
          }
          # --------------------------------
        }else if(i == tope & i == 3){
          expect[1:3] <- roxy$pos[www]
        }
        
        ##
      } # end of for loop
      #plot(x, type="l",ylim=c(0,800)); abline(v=expect, col="red", lty=3)
      ###########################################
      posi <- expect
      ## get rid of selected peaks after reaching the maximum values
      #tutu <- abs(length(x) - posi)
      #if(length(posi) > 3){
      #posi <- posi[1:which(tutu == min(tutu,na.rm = TRUE))]
      #}
      heii <- roxy$hei[which(roxy$pos %in% posi)]
      
      fact3 <- length(posi)/fact2
      
      if(length((posi)) < 6){ # good user
        fact <- summary(lm(ladder.call[1:length(posi)]~poly(posi, degree=length((posi))-1)))$r.squared * fact3
      }else{
        fact <- summary(lm(ladder.call[1:length(posi)]~poly(posi, degree=5)))$r.squared * fact3
      }
    
      #plot(ladder.call~posi)
      roxy2 <- list(pos=posi, hei=heii, wei=ladder.call[1:length(posi)], corr=abs(cor(ladder.call[1:length(posi)],posi)), error=fact)#sum(error, na.rm=TRUE))
    
      return(roxy2)
    }
    ############
    #----------- END OF CALLER
    ############
    ## end of caller
    # www <- data.frame(step4)[,1]
    #for(g in 1:dim(step4)[2]){
    #  plot(x,type="l",ylim=c(0,1000))
    #  abline(v=caller(www=step4[,g],roxy=roxy, ladder.call=ladder,x=x)$pos,col="red")
    #}
    
    rt <- apply(data.frame(step4), 2, FUN=caller, roxy=roxy, ladder.call=ladder,x=x)#
    corrs3 <- unlist(lapply(rt, function(x){x$error})) #; dis[which(dis == Inf)] <- 1
    #corrs <- unlist(lapply(rt, function(x){x[[4]]})) # extract correlations
    roxy3 <- rt[[which(corrs3 == max(corrs3))[1]]]
    #roxy3 <- rt[[5]]
    # plot(x, type="l",xlim=c(2000,7000), ylim=c(0,20000)); abline(v=roxy3$pos, col="red")
    if(draw == TRUE){
      limi <- sort(roxy3$hei, decreasing = TRUE)
      plot(x, type="l", xaxt="n", ylim=c(0,(limi[3]+1000)), cex.axis=0.6, las=2,  col=transp("grey35",0.7), ylab="RFU", xlab="", lwd=2, main=attributes(x)$mycomm, cex.main=cex.title)
      axis(1, at=roxy3$pos, labels=roxy3$wei, cex.axis=0.6)
      points(x=roxy3$pos, y=roxy3$hei,cex=1.1, col=transp("black",0.85))
      points(x=roxy3$pos, y=roxy3$hei, pch=20, col=transp("red",0.7))
      legend("topleft", legend=paste("Correlation:",round(roxy3$corr, digits=4), sep=""), bty="n")
      legend("topright", legend=c("Peaks selected"), col=c("red"), bty = "n", pch=c(20), cex=0.85)
      
    }
    roxy <- roxy3
    }
  }
  #####################
  return(roxy)
  #########
}

homo.panel <- function(x, panel, window=0.49){
  newxxx <- numeric()
  for(i in 1:length(x$wei)){
    differences <- abs(x$wei[i] - panel)
    v <- which(differences < window)[1]
    if(length(v) > 0){
      y <- panel[v]
    }else{y <- 0}
    newxxx[i] <- y
  }
  x$wei <- newxxx
  return(x)
}

read.abif <- function(filename, max.bytes.in.file = file.info(filename)$size, 
                      pied.de.pilote = 1.2, verbose = FALSE){
  #
  # Suppress warnings when reading strings with internal nul character:
  #
  RTC <- function(x, ...) suppressWarnings(rawToChar(x, ...))
  #
  # Define some shortcuts:
  #
  SInt32 <- function(f, ...) readBin(f, what = "integer", signed = TRUE, endian = "big", size = 4, ...)
  SInt16 <- function(f, ...) readBin(f, what = "integer", signed = TRUE, endian = "big", size = 2, ...)
  SInt8 <- function(f, ...) readBin(f, what = "integer", signed = TRUE, endian = "big", size = 1, ...)
  UInt32 <- function(f, ...) readBin(f, what = "integer", signed = FALSE, endian = "big", size = 4, ...)
  UInt16 <- function(f, ...) readBin(f, what = "integer", signed = FALSE, endian = "big", size = 2, ...)
  UInt8 <- function(f, ...) readBin(f, what = "integer", signed = FALSE, endian = "big", size = 1, ...)
  f32 <- function(f, ...) readBin(f, what = "numeric", size = 4,endian="little", ...)
  f64 <- function(f, ...) readBin(f, what = "numeric", size = 8, endian="little",...)
  #
  # Load raw data in memory:
  #
  fc <- file(filename, open = "rb")
  rawdata <- readBin(fc, what = "raw", n = pied.de.pilote*max.bytes.in.file,endian="little")
  if(verbose) print(paste("number of bytes in file", filename, "is", length(rawdata)))
  close(fc)
  #
  # Make a list to store results:
  #
  res <- list(Header = NULL, Directory = NA, Data = NA)
  #
  # Header section is 128 bytes long, located at a fixed position at the
  # beginning of the file. We essentially need the number of item and dataoffset
  #
  res$Header$abif <- RTC(rawdata[1:4])
  if(res$Header$abif != "ABIF") stop("file not in ABIF format")
  if(verbose) print("OK: File is in ABIF format")
  
  res$Header$version <- SInt16(rawdata[5:6])
  if(verbose) print(paste("File in ABIF version", res$Header$version/100))
  
  res$Header$DirEntry.name <- rawdata[7:10]
  if(verbose) print(paste("DirEntry name: ", RTC(res$Header$DirEntry.name)))
  
  res$Header$DirEntry.number <- SInt32(rawdata[11:14])
  if(verbose) print(paste("DirEntry number: ", res$Header$DirEntry.number))
  
  res$Header$DirEntry.elementtype <- SInt16(rawdata[15:16])
  if(verbose) print(paste("DirEntry elementtype: ", res$Header$DirEntry.elementtype))
  
  res$Header$DirEntry.elementsize <- SInt16(rawdata[17:18])
  if(verbose) print(paste("DirEntry elementsize: ", res$Header$DirEntry.elementsize))
  
  # This one is important:
  res$Header$numelements <- SInt32(rawdata[19:22])
  if(verbose) print(paste("DirEntry numelements: ", res$Header$numelements))
  
  # This one is important too:
  res$Header$dataoffset <- SInt32(rawdata[27:30])
  if(verbose) print(paste("DirEntry dataoffset: ", res$Header$dataoffset))
  dataoffset <- res$Header$dataoffset + 1 # start position is 1 in R vectors
  
  res$Header$datahandle <- SInt32(rawdata[31:34])
  if(verbose) print(paste("DirEntry datahandle: ", res$Header$datahandle))
  
  res$Header$unused <- SInt16(rawdata[35:128], n = 47)
  # Should be ingnored and set to zero
  res$Header$unused[1:length(res$Header$unused)] <- 0
  if(verbose) print(paste("DirEntry unused: ", length(res$Header$unused), "2-byte integers"))
  
  #
  # The directory is located at the offset specified in the header,
  # and consist of an array of directory entries.
  # We scan the directory to put values in a data.frame:
  #
  dirdf <- data.frame(list(name = character(0)))
  dirdf$name <- as.character(dirdf$name) # force to characters
  
  for(i in seq_len(res$Header$numelements)){
    deb <- (i-1)*res$Header$DirEntry.elementsize + dataoffset
    direntry <- rawdata[deb:(deb + res$Header$DirEntry.elementsize)]
    dirdf[i, "name"] <- RTC(direntry[1:4])
    dirdf[i, "tagnumber"] <- SInt32(direntry[5:8])
    dirdf[i, "elementtype"] <- SInt16(direntry[9:10])
    dirdf[i, "elementsize"] <- SInt16(direntry[11:12])
    dirdf[i, "numelements"] <- SInt32(direntry[13:16])
    dirdf[i, "datasize"] <- SInt32(direntry[17:20])
    dirdf[i, "dataoffset"] <- SInt32(direntry[21:24])
  }
  if(verbose){
    print("Element found:")
    print(dirdf$name)
  }
  #
  # Save Directory and make a list to store data:
  #
  res$Directory <- dirdf
  res$Data <- vector("list", nrow(dirdf))
  names(res$Data) <- paste(dirdf$name, dirdf$tagnumber, sep = ".")
  #
  # Data extraction:
  #
  for(i in seq_len(res$Header$numelements)){
    deb <- (i-1)*res$Header$DirEntry.elementsize + dataoffset
    # Short data are stored in dataoffset directly:
    if(dirdf[i, "datasize"] > 4){
      debinraw <- dirdf[i, "dataoffset"] + 1
    } else {
      debinraw <- deb + 20
    }
    elementtype <- dirdf[i, "elementtype"]
    numelements <- dirdf[i, "numelements"]
    elementsize <- dirdf[i, "elementsize"]
    data <- rawdata[debinraw:(debinraw + numelements*elementsize)]
    # unsigned 8 bits integer:
    if(elementtype == 1) res$Data[[i]] <- UInt8(data, n = numelements)
    # char or signed 8 bits integer
    if(elementtype == 2){
      res$Data[[i]] <- tryCatch(RTC(data),finally=paste(rawToChar(data,multiple=TRUE),collapse=""),error=function(er){cat(paste("an error was detected with the following  message:",er," but this error was fixed\n",sep=" "))})
    }
    
    # unsigned 16 bits integer:
    if(elementtype == 3) res$Data[[i]] <- UInt16(data, n = numelements)
    # short:
    if(elementtype == 4) res$Data[[i]] <- SInt16(data, n = numelements)
    # long:
    if(elementtype == 5) res$Data[[i]] <- SInt32(data, n = numelements)
    # float:
    if(elementtype == 7) res$Data[[i]] <- f32(data, n = numelements)
    # double:
    if(elementtype == 8) res$Data[[i]] <- f64(data, n = numelements)
    # date:
    if(elementtype == 10)
      res$Data[[i]] <- list(year = SInt16(data, n = 1), 
                            month = UInt8(data[-(1:2)], n = 1), day = UInt8(data[-(1:3)], n = 1))
    # time:
    if(elementtype == 11)
      res$Data[[i]] <- list(hour = UInt8(data, n = 1), 
                            minute = UInt8(data[-1], n = 1), second = UInt8(data[-(1:2)], n = 1),
                            hsecond = UInt8(data[-(1:3)], n = 1))
    # pString:
    if(elementtype == 18){
      n <- SInt8(rawdata[debinraw])
      pString <- RTC(rawdata[(debinraw+1):(debinraw+n)])
      res$Data[[i]] <- pString
    }
    # cString:
    if(elementtype == 19) res$Data[[i]] <- RTC(data[1:(length(data) - 1) ])
    # user:
    if(elementtype >= 1024) res$Data[[i]] <- data
    # legacy:
    if(elementtype %in% c(12, 13)) 
      warning("unimplemented legacy type found in file")
    if(elementtype %in% c(6, 9, 14, 15, 16, 17, 20, 128, 256, 384))
      warning("unsupported legacy type found in file")
  }
  return(res)
}

threshs <-
function(my.plant, min.thre=200, panel, ci=1.9){
  #my.plant <- new.whole.data[[1]]
  sta <- big.peaks.col(my.plant$yy, min.thre)
  sta$wei <- my.plant$xx[sta$pos]
  # subset only the ones participating in the panel
  hoho <- which(sta$wei < panel[2] & sta$wei > panel[1])
  if(length(hoho) > 0){
    sta <- list(pos=sta$pos[hoho], hei=sta$hei[hoho], wei= sta$wei[hoho])
    #plot(x=my.plant$xx, y=my.plant$yy, type="l", xlim=c(panel[1],panel[2]+50))
    #abline(h=mm);  abline(h=mm-se2);  abline(h=mm+se2)
    z <- which(sta$hei == max(sta$hei))
    mm <- mean(sta$hei)
    if(length(sta$hei) > 2){
      se2 <- ci * (sd(sta$hei[-z])/sqrt(length(sta$hei[-z])))
    }else{ se2 <- ci * (sd(sta$hei)/sqrt(length(sta$hei)))}
    
    
    newthre <- mm-se2
  }else{newthre <- 500}
  
  return(newthre)
}

jm.conv <- function(a){ 
  v1 <- seq(1, dim(a)[2], by=2)
  v2 <- v1+1
  a2 <- as.matrix(a)
  
  a3 <- apply(a2, 2, round)
  
  alls <- list(NA)
  for(i in 1:length(v1)){
    s1 <- v1[i]
    s2 <- v2[i]
    x <- num.to.lett(as.matrix(a3[,s1:s2]))
    alls[[i]] <- apply(x,2, letter.to.jm)
  }
  res <- data.frame(matrix(unlist(alls), ncol=length(alls)))
  names(res) <- names(a)[v1]
  return(res)
}

score.markers <- function (my.inds, channel = 1, n.inds = NULL, panel=NULL, shift=0.8, 
                        ladder, channel.ladder=NULL, ploidy=2, left.cond=c(0.6,3), 
                        right.cond=0.35, warn=FALSE, window=0.5, init.thresh=200, 
                        ladd.init.thresh=200, method="iter2", env = parent.frame(), 
                        my.palette=NULL,plotting=TRUE, electro=FALSE, pref=3) 
  {
    oldw <- getOption("warn")
    options(warn = -1)
    ci.upp=1.96; ci.low=1.96; dev=50 ;thresh=NULL
    if(length(n.inds) > length(my.inds)){
      print(paste("Hey! you are trying to examine more individuals than the ones you actually read? You selected in 'n.inds' argument", length(n.inds), "individuals but you only provided", length(my.inds), " individuals. Please select a number of individuals smaller or same size than the ones contained in 'my.inds' argument"))
      stop
    }else{
      cat(paste("\n1) You have used a shift of", shift, "base pairs. All peaks at that distance from the tallest peak will be ignored and be considered noise. \n2) In addition the window used is", window, ". Which means that all peaks closer by that distance to panel peaks will be accounted as peaks. \n3) Remember using the get.scores() function to extract the results from this output as a dataframe. \n\n"))
      #print(paste("In addition the window used is", window, ". Which means that all peaks closer by that distance to panel peaks will be accounted as peaks"))
    }
    if(method == "ci"){
      print(paste("Please make sure you have used the same 'dev' value you found convenient for your ladder detection or probably your call will not match"))
    }
    # the shift is used to declare when peaks will be considered the same peak and only the tallest will be picked
    # window willl be used to declase a peak the same for each base pair, i.e 170.4 is 170 same than 169.7
    ### makes a subset with the desired plants
    
    ## channel where the ladder is located
    if(is.null(channel.ladder )){
      channel.ladder <- dim(my.inds[[1]])[2]
    }else{ channel.ladder <- channel.ladder}
    
    if(dim(my.inds[[1]])[2] < channel.ladder){
      print(paste("ERROR MY FRIEND!! you have indicated an argument channel.ladder=5, but your data contains less channels/colors"))
      stop
    }
    ## number of samples to do
    if(is.null(n.inds )){
      n.inds <- c(1:length(my.inds))
    }else{n.inds <- n.inds}
    ## thresh
    if(is.null(thresh)){
      thresh <- rep(list(c(1,1,1,1,1)), length(my.inds))
    }else{thresh <- thresh}
    ####################
    ## initialize the progress bar
    count <- 0
    tot <- length(n.inds)
    pb <- txtProgressBar(style = 3)
    setTxtProgressBar(pb, 0)
    #####################
    
    my.inds2 <- list(NA)
    thresh2 <- list(NA)
    for (i in 1:length(n.inds)) {
      ###################
      count <- count + 1
      ##################
      v1 <- n.inds[i]
      my.inds2[[i]] <- my.inds[[v1]]
      names(my.inds2)[i] <- names(my.inds)[i]
      #################################
      setTxtProgressBar(pb, (count/tot)*.25)### keep filling the progress bar
      ################################
    }
    #my.inds <- my.inds2
    #if (!require("zoom")) {
    # install.packages("zoom")
    #  require("zoom")
    #}
    ncfp <- c("COLOR 1", "COLOR 2", "COLOR 3", "COLOR 4", "COLOR 5", "COLOR 6")
    if(!is.null(my.palette)){
      cfp <- rep(my.palette,100)
    }else{
      cfp <- c("cornflowerblue", "chartreuse4", "gold2", "red", "orange", "purple")
    }
    
    col.list <- list(NA)
    att1 <- numeric()
    #####################################################################################################
    # this part of the code finds the average height in all samples to be able to plot more uniformly
    #max.lis <- function(l1, colo){
    # takes a list containing a data frmae and returns the maximum value of the column provided in colo argument
    #  y <- max(l1[-c(1:1500),colo]); return(y)
    #}
    #maxi <- lapply(my.inds, max.lis, colo=channel)
    #pro <- mean(unlist(maxi))
    ######################################################################################################
    ### this part extracts all the models for each single plant in my plants
    list.data <- list(NA)
    if(exists("list.data.covarrubias")){
      list.data <- env$list.data.covarrubias
    }else{
      list.ladders <- lapply(my.inds2, function(x){y <- x[,channel.ladder]; return(y)})
      # extract ladder channels for all plants
      list.data <- lapply(list.ladders, find.ladder, ladder=ladder, ci.upp=ci.upp, ci.low=ci.low, draw=F, dev=dev, warn=warn, method=method,init.thresh=ladd.init.thresh)
    }
    # this models uses indexes and predicts base airs
    list.models <- lapply(list.data, function(da){y <- da[[3]]; x <- da[[1]];mod <- lm(y~ I(x) + I(x^2) + I(x^3) + I(x^4) + I(x^5), data=da); return(mod)})
    # this models uses pairs and predicts indexes
    list.models.inv <- lapply(list.data, function(da){x <- da[[3]]; y <- da[[1]];mod <- lm(y~ x, data=da); return(mod)})
    
    ######################################################################
    #########################################################################
    xx <- lapply(my.inds2, function(x, channel){1:length(x[,channel])}, channel=channel)
    newxx <- numeric()
    newyy <- numeric()
    new.whole.data <- list(NA)
    for(h in 1:length(xx)){
      h1 <- n.inds[h]
      ###################
      count <- count + 1
      ###################
      # h1 is used to make sure it matches if nplants is a weird vector
      newxx <- as.vector(predict(list.models[[h1]], newdata=data.frame(x=xx[[h]])))
      newyy <- my.inds2[[h]][,channel]
      new.whole.data[[h]] <- list(xx=newxx, yy=newyy)
      ################
      setTxtProgressBar(pb, (count/tot)*.25)### keep filling the progress bar
      ################################
    }
    
    top <- max(unlist(lapply(new.whole.data, function(x){max(x$yy)})))
    bott <- min(unlist(lapply(new.whole.data, function(x){min(x$yy)})))
    ### STARTS REDUCTION OF THE DATA ###
    list.weis <- list(NA) 
    lower.bounds <- numeric()
    for(k in 1:length(my.inds2)){
      #sh <- as.vector(predict(list.models.inv[[k]], newdata=data.frame(x=shift))) - list.models.inv[[k]]$coefficients[[1]]
      #if(length(panel) > 0){
      #pani <- c(min(panel)-1,  max(panel)+1)
      #maxpeak.in.pani <- max(new.whole.data[[k]][[2]][which(new.whole.data[[k]][[1]] > pani[1] & new.whole.data[[k]][[1]] < pani[2])])
      ## just create newthrsholds for plants who actually have peaks in that panel region
      #if( maxpeak.in.pani > init.thresh){
      # newtt <- threshs(my.plant=new.whole.data[[k]], min.thre=init.thresh, panel=pani, ci=thresh[[k]][channel])  
      #}else{newtt <- init.thresh}
      newtt <- init.thresh
      #####################################
      lower.bounds[k] <- newtt
      #}else{newtt <- init.thresh;  lower.bounds[k] <- newtt}
      plant <- big.peaks.col(new.whole.data[[k]]$yy, newtt)
      plant$wei <- new.whole.data[[k]]$xx[plant$pos]
      plant <- separate(plant, shift, type="bp")
      #plant$wei <- new.whole.data[[k]]$xx[plant$pos]
      list.weis[[k]] <- plant
      ###################
      if(plotting == TRUE){
        count <- count + 1
      }else{count <- count + 2}
      
      setTxtProgressBar(pb, (count/tot)*.25)### keep filling the progress bar
      
      ################################
    }
    # round digits
    list.weis <- lapply(list.weis, function(x){x$wei <- round(x$wei,digits=4); return(x)})
    names(list.weis) <- names(my.inds2)
    
    if(length(panel) > 0){
      list.weis <- lapply(list.weis, reals, panel=panel, shi=shift, ploidy=ploidy, left.cond=left.cond, right.cond=right.cond, window=window)
      list.weis2 <- lapply(list.weis, FUN=homo.panel, panel=panel, window=window)
    }else{
      list.weis2 <- list.weis
    }
    # END OF THE ANALYSIS
    
    if(plotting == TRUE){
      #### PLOTTING PART ###
      layout(matrix(1:pref,pref,1))
      #nom <- length(n.inds)/28
      #if(nom <= 4){ # define the layout for your plants
      #  layout(matrix(1:4, 4, 1))
      #}else{
      #  disi <- best.layout(round(nom))
      #  layout(matrix(1:((disi)[1]*disi[2]), disi[1], disi[2]))}
      ### start
      if(length(panel) > 0){
        xm <- round(min(panel, na.rm = TRUE)-10, digits=0)
        xl <- round(max(panel, na.rm = TRUE)+10, digits=0)
      }else{xm <- 0; xl <- max(ladder)}
      for(g in 1:length(n.inds)){
        hh4 <- n.inds[g]
        
        if(length(which(new.whole.data[[g]]$xx > xm & new.whole.data[[g]]$xx < xl)) > 0){
        mylim <- max(new.whole.data[[g]]$yy[which(new.whole.data[[g]]$xx > xm & new.whole.data[[g]]$xx < xl)], na.rm = TRUE) +100
        }else{mylim=1000}
        
        if(is.infinite(mylim)){mylim=1000}
        plot( new.whole.data[[g]]$xx, new.whole.data[[g]]$yy, type="l", col=cfp[channel], xaxt="n",
              xlim=c(xm,xl),  ylim=c(-200,mylim), ylab="Intensity", main=paste(ncfp[channel], "plant", hh4), 
              xlab=names(list.models)[hh4], lwd=2, las=2)
        axis(1,at=c(xm:xl), labels=xm:xl, cex.axis=0.8)
        rect(xleft=(list.weis2[[g]]$wei-window),ybottom=(bott-200), xright=(list.weis2[[g]]$wei+window), ytop=(top+1000), col=transp("lightpink",0.3), border=NA)
        abline(v=list.weis[[g]]$wei, lty=3, col="blue", cex=0.5)
        abline(v=list.weis2[[g]]$wei, lty=3, col="red", cex=0.5)
        abline(h=lower.bounds[g], lty=2, col="chocolate", cex=0.5)
        legend("topright", legend=c("Peak found", "Panel peak", "Panel window", "Minimum Detected"), col=c("blue", "red", transp("lightpink",0.3), "chocolate"), bty = "n", lty=c(3,3,1,3), lwd=c(1,1,3,1), cex=0.75)
        ###################
        count <- count + 1
        setTxtProgressBar(pb, (count/tot)*.25)### keep filling the progress bar
        ################################
      }
      ######################################################################
      ######################################################################
      if(electro==TRUE){
        if(length(n.inds) > 1){
          layout(matrix(1,1,1))
          forjet <- lapply(new.whole.data, function(x){x$yy})
          #forjet <- lapply(forjet, function(x) replace(x, is.infinite(x),0))
          #forjet <- lapply(forjet, function(x) replace(x, is.na(x),0))
          forjet2 <- matrix(unlist(forjet), ncol=length(new.whole.data), byrow=F)
          forjet2[which(forjet2 < 0)] <- 0
          #forjet2[1:5,]
          #forjet2[,32]
          
          jet.colors <- colorRampPalette(c("#00007F", "blue", "#007FFF", "cyan", "#7FFF7F", "yellow", "#FF7F00", "red"))
          palette <- jet.colors(25)
          image(forjet2, col=palette, xaxt="n", yaxt="n", main=paste("Electrogram for plants scored in color", channel))
          #image(rt, col=palette, xaxt = "n", yaxt = "n", "Electrogram for plants scored")
          #axis(side=3,at=seq(0,1,by=(1/(length(list.jet2)-1))),names(mydata), cex.axis=0.5) #above
          labb <- paste(rep("Plant", (dim(forjet2)[2])), 1:(dim(forjet2)[2]))
          axis(side=2,at=seq(1/(dim(forjet2)[2]),1, by=1/(dim(forjet2)[2])),labels=labb, las=2, cex.axis=0.5) #left
        }
      }
    }
    close(pb) # close the progress bar
    options(warn = oldw)
    
    # dt <- get.scores(list.weis2)
    # 
    # list.weis2$DF <- dt
    
    return(list.weis2)
  }

overview <-function (my.inds, channel = 1, n.inds = c(1:length(my.inds)), 
                     xlimi=c(min(ladder),max(ladder)), ladder, 
                     channel.ladder=dim(my.inds[[1]])[2], ploidy=2, 
                     dev=50, method="iter", 
                     init.thresh=200, ladd.init.thresh=200, warn=TRUE, 
                     my.palette=NULL, env = parent.frame()) 
{
  if(dim(my.inds[[1]])[2] < channel.ladder){
    print(paste("ERROR MY FRIEND!! you have indicated an argument channel.ladder=5, but your data contains less channels/colors"))
    stop
  }
  if(method == "ci"){
    print(paste("Please make sure you have used the same 'dev' value you found convenient for your ladder detection or probably your call will not match"))
  }
  
  ####################
  ## initialize the progress bar
  count <- 0
  tot <- length(n.inds)
  pb <- txtProgressBar(style = 3)
  setTxtProgressBar(pb, 0)
  #####################
  
  my.inds2 <- list(NA)
  for (i in 1:length(n.inds)) {
    v1 <- n.inds[i]
    my.inds2[[i]] <- my.inds[[v1]]
    names(my.inds2)[i] <- names(my.inds)[v1]
  }
  my.inds <- my.inds2
  #if (!require("zoom")) {
   # install.packages("zoom")
  #  require("zoom")
  #}
  ncfp <- c("COL1", "COL2", "COL3", "COL4", "COL5")
  if(!is.null(my.palette)){
    cfp <- rep(my.palette,100)
  }else{
    cfp <- c("cornflowerblue", "chartreuse4", "gold2", "red", "orange", "purple")
  }
  
  col.list <- list(NA)
  att1 <- numeric()
  #####################################################################################################
  # this part of the code finds the average height in all samples to be able to plot more uniformly
  #max.lis <- function(l1, colo){
  # takes a list containing a data frmae and returns the maximum value of the column provided in colo argument
  #  y <- max(l1[-c(1:1500),colo]); return(y)
  #}
  #maxi <- lapply(my.inds, max.lis, colo=channel)
  #pro <- mean(unlist(maxi))
  ######################################################################################################
  ### this part extracts all the models for each single plant in my plants
  list.data <- list(NA)
  if(exists("list.data.covarrubias")){
    list.data <- env$list.data.covarrubias
  }else{
    list.ladders <- lapply(my.inds, function(x){y <- x[,channel.ladder]; return(y)})
    # extract ladder channels for all plants
    list.data <- lapply(list.ladders, find.ladder, ladder=ladder, draw=F, dev=dev, warn=warn, method=method,init.thresh=ladd.init.thresh)
  } # this models uses indexes and predicts base airs
  list.models <- lapply(list.data, function(da){y <- da[[3]]; x <- da[[1]];mod <- lm(y~ I(x) + I(x^2) + I(x^3) + I(x^4) + I(x^5), data=da); return(mod)})
  # this models uses pairs and predicts indexes
  list.models.inv <- lapply(list.data, function(da){x <- da[[3]]; y <- da[[1]];mod <- lm(y~ x, data=da); return(mod)})
  
  ######################################################################
  #########################################################################
  xx <- lapply(my.inds2, function(x, channel){1:length(x[,channel])}, channel=channel)
  newxx <- numeric()
  newyy <- numeric()
  new.whole.data <- list(NA)
  for(h in 1:length(xx)){
    h1 <- n.inds[h]
    ###################
    count <- count + 1
    ###################
    newxx <- as.vector(predict(list.models[[h1]], newdata=data.frame(x=xx[[h]])))
    newyy <- my.inds2[[h]][,channel]
    new.whole.data[[h]] <- list(xx=newxx, yy=newyy) 
    ################################
    setTxtProgressBar(pb, (count/tot)*.5)### keep filling the progress bar
    ################################
  }
  # list.data constains the information for the ladder and picks the closest to lineup the plants
  common <- lapply(list.data, function(x, xlimi){mins <- abs(x$wei - xlimi[1]); y <- x$pos[which(mins == min(mins))]; return(y)}, xlimi=xlimi)
  # gets the maximum height for each plant
  heii <- lapply(my.inds2, function(x){max(x[,channel])[1]})
  # get total height for the plot
  tot.heii <- sum(unlist(heii), na.rm=T)
  # stablish a 1x1 layout matrix
  layout(matrix(1,1,1))
  # start the plot with the first plant
  nn <- n.inds
  plot(new.whole.data[[1]]$xx[-c(1:common[[1]])],y=new.whole.data[[1]]$yy[-c(1:common[[1]])], type="l", yaxt="n",
       xlim=c(xlimi[1],xlimi[2]), ylim=c(0, tot.heii), col=cfp[channel], xlab="Size in base pairs", ylab="Plants selected from bottom to top", xaxt="n", lwd=2)
  axis(1, at=seq(xlimi[1],xlimi[2], by=2), labels=seq(xlimi[1],xlimi[2], by=2))
  b <- sum(unlist(heii)[1]) 
  
  toput <- names(my.inds2)
  
  legend(x=xlimi[1],y=b, legend=paste("Plant",toput[1]), bty="n", cex=0.65)
  count <- count + 1
  # make a loop for adding the lines of the other plants
  if(length(n.inds) > 1){
    for(i in 2:length(my.inds2)){
      hh1 <- n.inds[i]
      ###################
      count <- count + 1
      ###################
      a <- sum(unlist(heii)[1:(i-1)]) # maximum height of prevoius plant, bottom
      b <- sum(unlist(heii)[1:i]) # maximim height adding the new plant
      yy <- new.whole.data[[i]]$yy + a
      lines(new.whole.data[[i]]$xx[-c(1:common[[i]])], y=yy[-c(1:common[[i]])], type="l", col=cfp[channel],lwd=2)
      legend(x=xlimi[1],y=b, legend=paste("Plant",toput[i]), bty="n", cex=0.65)
      #plot(cc[-c(1:common[[i]])], , ylim=c(0, tot.heii))
      ################################
      setTxtProgressBar(pb, (count/tot)*.5)### keep filling the progress bar
      ################################
    }
  }
  close(pb) # close the progress bar
  return(names(my.inds2))
}

big.peaks.col <-
function(x, tre){
  r <- rle(x)
  v <- which(rep(x = diff(sign(diff(c(-Inf, r$values, -Inf)))) == -2, times = r$lengths))
  pos <- v[which(x[v] > tre)] #position of the real peaks
  hei <- x[pos]
  out <- list(pos=pos,hei=hei)
  return(out)
}

lapply_pb <- function(X, FUN, ...)
{
  env <- environment()
  pb_Total <- length(X)
  counter <- 0
  pb <- txtProgressBar(min = 0, max = pb_Total, style = 3)   
  
  # wrapper around FUN
  wrapper <- function(...){
    curVal <- get("counter", envir = env)
    assign("counter", curVal +1 ,envir=env)
    setTxtProgressBar(get("pb", envir=env), curVal +1)
    FUN(...)
  }
  res <- lapply(X, wrapper, ...)
  close(pb)
  res
}

letter.to.jm <- function (x) 
{
  allelesf <- na.omit(unique(x))
  all1 <- table(na.omit((x)))/length(x)
  v <- which(all1 > 0.1)
  allelesff <- names(all1)[v]
  v2 <- which(as.vector(unlist(allelesf)) %in% allelesff)
  alleles <- as.vector(unlist(allelesf))[v2]
  if (length(alleles) == 1) {
    alleles1 <- sort(alleles)
    y <- rep(NA, length(x))
    y[which(x == alleles1[1])] <- "-"
    y[which(is.na(x))] <- "--"
  }
  if (length(alleles) == 2) {
    alleles1 <- alleles
    if ((alleles1[1] == "A") | (alleles1[1] == "C") | (alleles1[1] == 
                                                       "T") | (alleles1[1] == "G")) {
      y <- rep(NA, length(x))
      y[which(x == alleles1[1])] <- "nn"
      y[which(x == alleles1[2])] <- "np"
      y[which(is.na(x))] <- "--"
    }
    if ((alleles1[1] == "R") | (alleles1[1] == "Y") | (alleles1[1] == 
                                                       "S") | (alleles1[1] == "W") | (alleles1[1] == "K") | 
        (alleles1[1] == "M")) {
      y <- rep(NA, length(x))
      y[which(x == alleles1[1])] <- "lm"
      y[which(x == alleles1[2])] <- "ll"
      y[which(is.na(x))] <- "--"
    }
  }
  if (length(alleles) == 3) {
    yuyu <- c(length(which(x == alleles[1])), length(which(x == 
                                                             alleles[2])), length(which(x == alleles[3])))
    names(yuyu) <- alleles
    alleles1 <- names(sort(yuyu, decreasing = T))
    y <- rep(NA, length(x))
    y[which(x == alleles1[1])] <- "hk"
    y[which(x == alleles1[2])] <- "hh"
    y[which(x == alleles1[3])] <- "kk"
    y[which(is.na(x))] <- "--"
  }
  if (length(alleles) == 4) {
    alleles1 <- alleles
    # R = A:G
    # K = G:T
    # A = A:A
    # W = A:T
    #alleles2 <- sort(alleles)
    mat <- alleles1[which(alleles1 %in% x[1])]
    pat <- alleles1[which(alleles1 %in% x[2])]
    
    #oc <- c(alleles1[1], alleles1[2], alleles2[1])
    #v <- which(alleles1 == oc)
    #v2 <- 1:4
    #v3 <- which(v2 != v)
    y <- rep(NA, length(x))
    y[which(x == mat)] <- "ef"
    y[which(x == pat)] <- "eg"
    y[which(x == "A")] <- "ee"
    y[which(x == "K")] <- "fg"
    y[which(is.na(x))] <- "--"
  }
  if (length(alleles) == 6) {
    alleles1 <- alleles
    if (alleles1[1] == "R") {
      y <- rep(NA, length(x))
      y[which(x == "M")] <- "ac"
      y[which(x == "W")] <- "ad"
      y[which(x == "S")] <- "bc"
      y[which(x == "K")] <- "bd"
      y[which(x == "R")] <- "ab"
      y[which(x == "Y")] <- "cd"
      y[which(is.na(x))] <- "--"
    }
    if (alleles1[1] == "Y") {
      y <- rep(NA, length(x))
      y[which(x == "M")] <- "ac"
      y[which(x == "W")] <- "bc"
      y[which(x == "S")] <- "ad"
      y[which(x == "K")] <- "bd"
      y[which(x == "Y")] <- "ab"
      y[which(x == "R")] <- "cd"
      y[which(is.na(x))] <- "--"
    }
    if (alleles1[1] == "S") {
      y <- rep(NA, length(x))
      y[which(x == "M")] <- "ac"
      y[which(x == "Y")] <- "ad"
      y[which(x == "R")] <- "bc"
      y[which(x == "K")] <- "bd"
      y[which(x == "S")] <- "ab"
      y[which(x == "W")] <- "cd"
      y[which(is.na(x))] <- "--"
    }
    if (alleles1[1] == "W") {
      y <- rep(NA, length(x))
      y[which(x == "M")] <- "ac"
      y[which(x == "Y")] <- "bc"
      y[which(x == "R")] <- "ad"
      y[which(x == "K")] <- "bd"
      y[which(x == "W")] <- "ab"
      y[which(x == "S")] <- "cd"
      y[which(is.na(x))] <- "--"
    }
    if (alleles1[1] == "K") {
      y <- rep(NA, length(x))
      y[which(x == "R")] <- "ac"
      y[which(x == "S")] <- "ad"
      y[which(x == "W")] <- "bc"
      y[which(x == "Y")] <- "bd"
      y[which(x == "K")] <- "ab"
      y[which(x == "M")] <- "cd"
      y[which(is.na(x))] <- "--"
    }
    if (alleles1[1] == "M") {
      y <- rep(NA, length(x))
      y[which(x == "R")] <- "ac"
      y[which(x == "S")] <- "bc"
      y[which(x == "W")] <- "ad"
      y[which(x == "Y")] <- "bd"
      y[which(x == "M")] <- "ab"
      y[which(x == "K")] <- "cd"
      y[which(is.na(x))] <- "--"
    }
  }
  y[which(is.na(y) == TRUE)] <- "--"
  return(y)
}

num.to.lett <- function (xx) 
{
  s1 <- seq(1, 5000, by = 2)
  s2 <- seq(2, 5000, by = 2)
  mark.list <- list(NA)
  names.list <- list(NA)
  for (i in 1:(dim(xx)[2]/2)) {
    v1 <- s1[i]
    v2 <- s2[i]
    names.list[[i]] <- names(xx)[v1]
    xxx <- xx[, v1:v2]
    wer <- unique(which(is.na(xxx), arr.ind=T)[,1])
    if(length(wer) > 0){
      geno <- table(paste(xxx[-wer, 1], xxx[-wer, 2], sep = "-"))/sum(table(paste(xxx[-wer, 
                                                                                      1], xxx[-wer, 2], sep = "-")))
    }else{
      geno <- table(paste(xxx[, 1], xxx[, 2], sep = "-"))/sum(table(paste(xxx[, 
                                                                              1], xxx[, 2], sep = "-")))
    }
    geno <- sort(geno, decreasing = T)
    v <- which(geno < 0.1)
    if (length(v) > 0) {
      geno <- geno[-v]
    }
    alle <- table(c(xxx[, 1], xxx[, 2], sep = "-"))/sum(table(c(xxx[, 
                                                                    1], xxx[, 2], sep = "-")))
    alle <- sort(alle, decreasing = T)
    v <- which(alle < 0.1)
    if (length(v) > 0) {
      alle <- alle[-v]
    }
    n.all <- length(names(alle))
    alls <- as.numeric(names(alle))
    config <- paste(xxx[, 1], xxx[, 2], sep = "-")
    y <- vector(mode = "character", length = length(config))
    if (length(alls) == 2 & length(geno) == 2) {
      y[which(config == paste(alls[1], alls[2], sep = "-") | 
                config == paste(alls[2], alls[1], sep = "-"))] <- "R"
      y[which(config == paste(alls[1], alls[1], sep = "-"))] <- "A"
      y[which(config == paste(alls[2], alls[2], sep = "-"))] <- "A"
    }
    if (length(alls) == 2 & length(geno) == 3) {
      y[which(config == paste(alls[1], alls[1], sep = "-"))] <- "A"
      y[which(config == paste(alls[2], alls[1], sep = "-") | 
                config == paste(alls[1], alls[2], sep = "-"))] <- "R"
      y[which(config == paste(alls[2], alls[2], sep = "-"))] <- "G"
    }
    if (length(alls) == 3 & length(geno) == 4) {
      y[which(config == paste(alls[1], alls[1], sep = "-"))] <- "A"
      y[which(config == paste(alls[2], alls[1], sep = "-") | 
                config == paste(alls[1], alls[2], sep = "-"))] <- "R"
      y[which(config == paste(alls[3], alls[1], sep = "-") | 
                config == paste(alls[1], alls[3], sep = "-"))] <- "W"
      y[which(config == paste(alls[3], alls[2], sep = "-") | 
                config == paste(alls[2], alls[3], sep = "-"))] <- "K"
    }
    if (length(alls) == 4) {
      y[which(config == paste(alls[1], alls[1], sep = "-"))] <- "A"
      y[which(config == paste(alls[2], alls[2], sep = "-"))] <- "T"
      y[which(config == paste(alls[3], alls[3], sep = "-"))] <- "C"
      y[which(config == paste(alls[4], alls[4], sep = "-"))] <- "G"
      y[which(config == paste(alls[2], alls[1], sep = "-") | 
                config == paste(alls[1], alls[2], sep = "-"))] <- "W"
      y[which(config == paste(alls[3], alls[1], sep = "-") | 
                config == paste(alls[1], alls[3], sep = "-"))] <- "M"
      y[which(config == paste(alls[4], alls[1], sep = "-") | 
                config == paste(alls[1], alls[4], sep = "-"))] <- "R"
      y[which(config == paste(alls[2], alls[3], sep = "-") | 
                config == paste(alls[3], alls[2], sep = "-"))] <- "Y"
      y[which(config == paste(alls[2], alls[4], sep = "-") | 
                config == paste(alls[4], alls[2], sep = "-"))] <- "K"
      y[which(config == paste(alls[3], alls[4], sep = "-") | 
                config == paste(alls[3], alls[4], sep = "-"))] <- "S"
    }
    y[which(y == "")] <- NA
    mark.list[[i]] <- y
  }
  xx2 <- data.frame(matrix(unlist(mark.list), nrow = dim(xx)[1]))
  rownames(xx2) <- rownames(xx)
  names(xx2) <- unlist(names.list)
  return(xx2)
}

overview2 <- function (my.inds, channel = 1, ladder, xlim = NULL, ylim = NULL, n.inds = NULL, channel.ladder = NULL, ploidy = 2,  
                       method="iter2", init.thresh=NULL, ladd.init.thresh=200, lwd=.25, warn=TRUE, min.panel=100, 
                       suggested=TRUE, env = parent.frame(), my.palette=NULL, verbose=TRUE) 
{
  #
  layout(matrix(1,1,1))
  if(length(channel) > 1 & is.null(ylim)){
    #print("yu")
    limosna <- c(min(unlist(lapply(my.inds, min, na.rm=TRUE)), na.rm=TRUE),
                 max(unlist(lapply(my.inds, max, na.rm=TRUE)), na.rm=TRUE)
    )
  }else{
    limosna <- ylim
  }
  
  suggested.list <- list()
  counter <- 0
  for(hhh in channel){
    counter <- counter+1
    cols <- hhh
    dev = 50
    oldw <- getOption("warn")
    options(warn = -1)
    #options(show.error.messages = FALSE)
    if(method == "ci"){
      print(paste("Please make sure you have used the same 'dev' value you found convenient for your ladder detection or probably your call will not match"))
    }
    ## channel where the ladder is located
    if(is.null(channel.ladder )){
      channel.ladder <- dim(my.inds[[1]])[2]
    }else{ channel.ladder <- channel.ladder}
    if(dim(my.inds[[1]])[2] < channel.ladder){
      print(paste("ERROR MY FRIEND!! you have indicated an argument channel.ladder=5, but your data contains less channel/colors"))
      stop
    }
    ## provide initial threshold if not specified
    if(is.null(init.thresh) & suggested){
      listaaaa<- do.call("cbind",lapply(my.inds, function(x){y <- x[,cols]; return(y)}))
      init.thresh <-quantile(listaaaa,.99)#median(listaaaa)*10
    }
    
    ## number of samples to do
    if(is.null(n.inds )){
      n.inds <- c(1:length(my.inds))
    }else{n.inds <- n.inds}
    ## x limits
    if(is.null(xlim )){
      xlim <- c(min(ladder), max(ladder))
    } else{ xlim <- xlim}
    ####################
    ## initialize the progress bar
    count <- 0
    tot <- length(n.inds)
    pb <- txtProgressBar(style = 3)
    setTxtProgressBar(pb, 0)
    #####################
    
    my.inds2 <- list(NA)
    for (i in 1:length(n.inds)) {
      v1 <- n.inds[i]
      my.inds2[[i]] <- my.inds[[v1]]
      names(my.inds2)[i] <- names(my.inds)[i]
    }
    my.inds <- my.inds2
    ncfp <- c("COL1", "COL2", "COL3", "COL4", "COL5")
    if(!is.null(my.palette)){
      cfp <- rep(my.palette,100)
    }else{
      cfp <- c("cornflowerblue", "chartreuse4", "gold2", "red", 
               "orange", "purple")
    }
    
    col.list <- list(NA)
    att1 <- numeric()
    ############################################
    list.data <- list(NA)
    if(exists("list.data.covarrubias")){
      list.data <- env$list.data.covarrubias
    }else{
      list.ladders <- lapply(my.inds, function(x){y <- x[,channel.ladder]; return(y)})
      # extract ladder channel for all plants
      list.data <- lapply(list.ladders, find.ladder, ladder=ladder, draw=F, dev=dev, warn=warn, method=method,init.thresh=ladd.init.thresh)
    } # this models uses indexes and predicts base airs
    list.models <- lapply(list.data, function(da){y <- da[[3]]; x <- da[[1]];mod <- lm(y~ I(x) + I(x^2) + I(x^3) + I(x^4) + I(x^5), data=da); return(mod)})
    # this models uses pairs and predicts indexes
    list.models.inv <- lapply(list.data, function(da){x <- da[[3]]; y <- da[[1]];mod <- lm(y~ x, data=da); return(mod)})
    ##############################################
    xx <- lapply(my.inds2, function(x, cols) {
      1:length(x[, cols])
    }, cols = cols)
    newxx <- numeric()
    newyy <- numeric()
    new.whole.data <- list(NA)
    for (h in 1:length(xx)) {
      h1 <- n.inds[h]
      ###################
      count <- count + 1
      ###################
      newxx <- as.vector(try(predict(list.models[[h1]], newdata = data.frame(x = xx[[h]])), silent = TRUE))
      #newxx <- as.vector(predict(list.models[[h1]], newdata = data.frame(x = xx[[h]])))
      newyy <- my.inds2[[h]][, cols]
      new.whole.data[[h]] <- list(xx = newxx, yy = newyy)
      ################################
      setTxtProgressBar(pb, (count/tot))### keep filling the progress bar
      ################################
    }
    common <- lapply(list.data, function(x, xlim) {
      mins <- abs(x$wei - xlim[1])
      y <- x$pos[which(mins == min(mins))][1]
      return(y)
    }, xlim = xlim)
    heii <- lapply(my.inds2, function(x) {
      max(x[, cols])[1]
    })
    
    ## ---------------------------------------
    ## provide inital guesses of which should be the panel peaks
    if(suggested){
      my.panel <- lapply(new.whole.data,
                         function(popo){
                           pann <- big.peaks.col(popo$yy, tre=init.thresh)
                           pann2 <- popo$xx[pann$pos]
                           pann3 <- list(pos=pann$pos, hei=pann$hei, wei=pann2)
                           pkpn <- separate(pann3, type="bp", shift=1)
                           return(list(wei=pkpn$wei, hei=pkpn$hei))
                         }
      )
      #plot(unlist(my.panel))
      ## unlist all the peaks found for all the plants
      allpan <- unlist(lapply(my.panel, function(x){x$wei}))
      allhei <- unlist(lapply(my.panel, function(x){x$hei}))
      ## create a vector to store a the good peaks
      panel1.1 <- numeric()
      heis1.1 <- numeric()
      for(za in seq(1,500, by=1)){
        step1 <- abs(za- allpan)
        good <- which(step1 < 0.48) # peak present at with minumum error of x bp
        if(length(good) > (length(n.inds)*.05)){ # more than 20% of the times present
          panel1.1[za] <- mean(allpan[good])
          heis1.1[za] <- mean(allhei[good])
        }else{panel1.1[za] <- NA; heis1.1[za] <- NA}
      }
      if(is.null(xlim)){
        panel.sugg <- panel1.1[-which(panel1.1 < min.panel | is.na(panel1.1))]
        heis.sugg <- heis1.1[-which(panel1.1 < min.panel | is.na(panel1.1))]
      }else{
        prov <- panel1.1[which(panel1.1 > xlim[1] & panel1.1 < xlim[2])]
        bad <- which(is.na(prov))
        if(length(bad) > 0){panel.sugg <- prov[-bad]}else{panel.sugg <- prov}
        #--
        prov2 <- heis1.1[which(panel1.1 > xlim[1] & panel1.1 < xlim[2])]
        bad2 <- which(is.na(prov2))
        if(length(bad2) > 0){heis.sugg <- prov[-bad2]}else{heis.sugg <- prov2}
      }
    }
    ## ------------------------------------------
    ## parameters for plots and lines
    tot.heii <- max(unlist(heii), na.rm = T)
    ## ylims defaults
    if(is.null(ylim )){
      ylim <- c(0, tot.heii)
    }else{ylim <- ylim}
    ##
    layout(matrix(1, 1, 1))
    nn <- n.inds
    
    if(length(channel) > 1){ # if user is trying to see more than one channel
      ylim[1] <- limosna[1]
      ylim[2] <- limosna[2]
    }
    
    plot(new.whole.data[[1]]$xx[-c(1:common[[1]])], y = new.whole.data[[1]]$yy[-c(1:common[[1]])], 
         type = "l", xlim = c(xlim[1], xlim[2]), 
         ylim=c(ylim[1],ylim[2]), yaxt = "n", col = transp(cfp[cols],0.6), xlab = "Size in base pairs", 
         ylab = "DNA intensity in RFU", xaxt = "n", 
         lwd = lwd)
    axis(1, at = seq(xlim[1], xlim[2], by = 2), labels = seq(xlim[1], 
                                                             xlim[2], by = 2), cex.axis=0.7)
    axis(2, at = seq(0, tot.heii, by = 500), labels = seq(0, tot.heii, by = 500), las=1, cex.axis=0.4)
    if(length(n.inds) == 1){
      count <- count + 50
      setTxtProgressBar(pb, (count/tot))
    }else{count <- count + 1}
    
    #b <- sum(unlist(heii)[1])
    #legend(x = xlim[1], y = b, legend = paste("Plant", nn[1]), 
    #      bty = "n")
    if (length(n.inds) > 1) {
      for (i in 2:length(my.inds2)) {
        ###################
        count <- count + 1
        ###################
        a <- sum(unlist(heii)[1:(i - 1)])
        b <- sum(unlist(heii)[1:i])
        yy <- new.whole.data[[i]]$yy
        lines(new.whole.data[[i]]$xx[-c(1:common[[i]])], 
              y = yy[-c(1:common[[i]])], type = "l", col = transp(cfp[cols],0.6), 
              lwd = lwd)
        #legend(x = xlim[1], y = b, legend = paste("Plant", 
        #                                          nn[i]), bty = "n")
        ################################
        setTxtProgressBar(pb, (count/tot))### keep filling the progress bar
        ################################
      }
      #close(pb)
    }
    legend("topright", legend="Peaks suggested", pch=20, col="red", bty="n", cex=0.75)
    
    #setTxtProgressBar(pb, (tot/tot)*.5)
    
    options(warn = oldw)
    close(pb) # close the progress bar
    if(verbose){
    cat("\n THE PEAKS RETURNED ARE SUGGESTIONS. \n   What you should do: \n a) Use the locator function, i.e. ''my.panel <- locator(type='p', pch=20, col='red')$x'' \n b) Click over the peaks you want to include in your panel \n c) Press the 'esc' key when done selecting peaks \n d) Make sure to provide the panel vector in the score.easy() function \n \n")
    }
    if(suggested){
      points(x=panel.sugg, y=heis.sugg, pch=20, cex=0.7, col="red")
      points(x=panel.sugg, y=heis.sugg, cex=0.9, col="black")
      #return(panel.sugg)
      suggested.list[[counter]] <- panel.sugg
    }
    
    par(new=TRUE)
    verbose=FALSE # after the first color stop returning the messages
  }# end of the loop for each color
  
  par(new=FALSE)
  
  if(suggested){
    names(suggested.list) <- paste("channel_",channel,sep="")
    return(suggested.list)
  }
  
  layout(matrix(1,1,1))
  #plot(0, col=transp("white",0), xlab="",ylab = "", xaxt="n", yaxt="n")
  #par(las=0)
  
}

saturate <- function (y) {
  y2 <- y
  st1 <- which(y > 8000)
  ######
  if (length(st1) > 1) { ## 1st condition
    peak.sta <- c(1, which(diff(st1) > 12) + 1)
    peak.end <- c(which(diff(st1) > 12), length(st1))
    ini <- st1[peak.sta]
    end <- st1[peak.end]
    "%!in%" <- function(x, y) !(x %in% y)
    for (i in 1:length(ini)) { ### for loop
      v1 <- ini[i]
      v2 <- end[i]
      v3 <- v1:v2
      v4 <- v3[big.peaks.col(y[v3], tre = 7000)$pos]
      heis <- y[v4]
      #abline(v=v4, col="red", lty=3)
      ### if peaks found are 2
      
      if (length(v4) >= 2) { # 2nd condition
        sort.heis <- sort(heis, decreasing=TRUE)[1:2]
        v4.1 <- sort(v4[which(heis %in% sort.heis)], decreasing=FALSE)
        # now do the same than before
        v5b <- (v4.1[1]:(v4.1[length(v4.1)]))
        v5 <- v5b[which(v5b %!in% v4.1)]
        for (j in 1:length(v5)) {
          v6 <- v5[j]
          left <- v4.1[1]
          right <- v4.1[2]
          a <- y[left] - y[v6]
          b <- y[right] - y[v6]
          #print(a);print(b);  
          #print(y[v6] + (2 * ((abs(a) + abs(b))/2)))
          #if (a < 0 & b < 0) {
          y2[v6] <- y[v6] + (2 * ((abs(a) + abs(b))/2))
          #}
        }
      }else{ # else to 2nd condition
        y2 <- y2 # cannot be reseted
      }   
    }
  }  else {### else to the first condition
    y2 <- y2
  }
  return(y2)
}

get.scores <-
function(my.scores, mark="mark"){
  y <- lapply(my.scores, function(x){x$wei})
  n <- lapply(my.scores, function(x){length(x$wei)})
  nn <- max(unlist(n))
  da <- data.frame(matrix(NA, byrow=T, ncol=nn, nrow=length(my.scores)))
  rownames(da) <- names(my.scores)
  for(i in 1:dim(da)[1]){
    da[i, 1:(length(y[[i]]))] <- y[[i]]
  }
  col.na <- vector(mode="character")
  for(k in 1:nn){
    col.na[k] <- paste(mark,"A.",k,sep="")
  }
  names(da) <- col.na
  return(da)
}

storing.inds <-
  function (folder, channels=NULL, fourier=TRUE, saturated=TRUE, lets.pullup=TRUE, 
            plotting=FALSE, rawPlot=FALSE, llength=3000, ulength=80000) {
    #if (!require("seqinr")) {
    #  install.packages("seqinr")
    #  require("seqinr")
    #}
    #storing the names of the files
    coli <-  c("cornflowerblue", "chartreuse4", "gold2", "red", 
               "orange", "purple")
    setwd(paste(folder))
    listp2 <- dir(folder, "*.fsa")
    FSA <- TRUE
    
    if(length(listp2)==0){
      listp2 <- dir(folder, "*.txt")
      if(length(listp2)==0){
        stop(paste("We have not found files with extension .fsa or .txt. Please\nmake sure this is the right folder:", folder,"\n"), call. = FALSE)
      }
      FSA=FALSE
    }
    # list to store all matrices of 4 columns
    all.inds.mats <- list(NA)
    
    cat("\n")
    if(FSA){
      print("Reading FSA files")
    }else{
      print("Reading files")
    }
    ####################
    ## initialize the progress bar
    count <- 0
    tot <- length(listp2)
    pb <- txtProgressBar(style = 3)
    setTxtProgressBar(pb, 0)
    cfound <- NULL
    #####################
    
    if(FSA){ # if fsa files were read
      for(i in 1:length(listp2)){
        ###################
        count <- count + 1
        ###################
        fsaFile<-read.abif(listp2[i]) # file read
        lens <- lapply(fsaFile$Data, length)
        aaa <- table(unlist(lens)) # length of vectors in data
        #### CONDITION IF CHANNELS WAS NOT SPECIFIED
        if(is.null(channels)){ # find number of channels
          channels <- as.vector(aaa[which(as.numeric(names(aaa)) > llength & as.numeric(names(aaa)) < ulength)])
        }
        ### if we found two possible places where data is stored
        if(length(channels) > 1 & is.null(cfound)){
          cat("\nYour data has two possible places where run markers could be stored \nand we don't know which is the correct one\n")#, call. = FALSE)
          prov <- aaa[which(as.numeric(names(aaa)) > llength & as.numeric(names(aaa)) < ulength)]
          prov2 <- matrix(prov, nrow=1)
          rownames(prov2) <- "number.of.channels.found"; 
          colnames(prov2) <- paste("Run_Length",names(prov),"indexes",sep="_")
          prov2 <- rbind(prov2,prov2)
          rownames(prov2)[2] <- "number.to.type.if.selected"
          prov2[2,1:ncol(prov2)] <- 1:ncol(prov2) 
          cat("Please tell us which is the right place where the channels are located\n\n")
          print(prov2)
          inut <- as.numeric(readline(prompt="Enter one of the number.to.type: "))
          channels <- channels[inut]
          cfound <- channels
        }
        #################
        
        real.len <- as.numeric(names(aaa)[which(aaa == channels & as.numeric(names(aaa)) > llength & as.numeric(names(aaa)) < ulength )]) # length of the data elements that meet the requirement to be a data file
        v <- as.vector(which(unlist(lens) == real.len)) # length to find, elements storing the data found
        reads<-list(NA) # to store info
        for(j in 1:channels){ # for each channel
          v2 <- v[j]
          reads[[j]] <- fsaFile$Data[[v2]]
        }
        prov <- matrix(unlist(reads),ncol=channels)
        colnames(prov) <- paste("channel_",1:ncol(prov),sep="")
        rownames(prov) <- paste("index_",1:nrow(prov),sep="")
        all.inds.mats[[i]] <- prov
        prov <- NULL
        names(all.inds.mats)[i] <- as.character(listp2[i])
        #attributes(all.inds.mats[[i]]) <- list(FSA=FALSE)
        #attributes(list.ladders[[t]]) <- list(mycomm=names(list.ladders)[t])
        ################################
        setTxtProgressBar(pb, count/tot)### keep filling the progress bar
        ################################
      }
      #attributes(all.inds.mats) <- list(FSA=TRUE)
    }else{ # cqs files
      for(i in 1:length(listp2)){
        ###################
        count <- count + 1
        ###################
        
        # read the 7th line where the line with column names is located
        dodo <-scan(listp2[i],what="a",nlines=7,skip=6,quiet=TRUE) 
        chaco <- grep("FILTER",dodo)# which have name filter
        nchaco <- length(chaco) # number of channels run
        topick <- chaco[1]:(chaco[1] + (nchaco-1)) # where are the lines to read
        
        ds <- read.table(listp2[i], header = FALSE, skip=84)
        all.inds.mats[[i]] <- as.matrix(ds[,topick])
        names(all.inds.mats)[i] <- as.character(listp2[i])
        #attributes(all.inds.mats[[i]]) <- list(FSA=FALSE)
        ################################
        setTxtProgressBar(pb, count/tot)### keep filling the progress bar
        ################################
      }
      #attributes(all.inds.mats[[i]]) <- list(FSA=FALSE)
    }
    
    close(pb) # close the progress bar
    if(fourier == TRUE){ #FOURIER
      print("Applying Fourier tranformation for smoothing...")
      all.inds.mats <- lapply_pb(all.inds.mats, function(x){apply(x, 2, transfft)})
    }
    if(saturated == TRUE){ #SATURATION
      print("Checking and correcting for saturated peaks...")
      all.inds.mats <- lapply_pb(all.inds.mats, function(x){apply(x, 2, saturate)})
    }
    if(FSA){
      if(lets.pullup==TRUE){#PULL UP
        print("Applying pull up correction to the samples to decrease noise from channel to channel")
        if(plotting == TRUE){
          all.inds.mats <- lapply_pb(all.inds.mats, pullup, channel=channels, plotting=TRUE)
        }
        all.inds.mats <- lapply_pb(all.inds.mats, pullup, channel=channels)
      }
    }
    ### ------------------
    ### ------------------
    
    if(rawPlot == TRUE){
      layout(matrix(1:2,2,1))
      coli <- cfp <- c("cornflowerblue", "chartreuse4", "gold2", "red", "orange", "purple")
      naname <- c("FAM","HEX","NED","ROX","LIZ")
      
      print("Plotting raw data")
      
      ####################
      ## initialize the progress bar
      count <- 0
      tot <- length(listp2)
      pb <- txtProgressBar(style = 3)
      setTxtProgressBar(pb, 0)
      #####################
      for(i in 1:dim(all.inds.mats[[1]])[2]){
        plot(all.inds.mats[[1]][,i], col=transp(coli[i],0.6), type="l", ylab="RFU", main=paste(naname[i],"channel. Plot",i,"of",dim(all.inds.mats[[1]])[2]), las=2)
        rect(par("usr")[1],par("usr")[3],par("usr")[2],par("usr")[4],col = "grey75")
        
        if(length(all.inds.mats) > 1){
          for(j in 1:length(all.inds.mats)){
            ###################
            count <- count + (1/dim(all.inds.mats[[1]])[2])
            ###################
            lines(all.inds.mats[[j]][,i], col=transp(coli[i],0.4), lwd=.2)
            ################################
            setTxtProgressBar(pb, count/tot)### keep filling the progress bar
            ################################
          }
        }
      }
      close(pb) # close the progress bar
    }
    
    
    layout(matrix(1,1,1))
    
    class(all.inds.mats)<-c("fsa_stored")
    
    cat("\n     Output is a LIST where each element of the list is a DATAFRAME \n            with the channels in columns for each FSA file.\n\n")
    return(all.inds.mats)
  }

plot.fsa_stored <- function(x, lay=c(2,1), channel=NULL, cex.legend=.5, ncol.legend=4,lims=NULL, color=NULL, ...) {
  digits = max(3, getOption("digits") - 3)
  transp <- function (col, alpha = 0.5){
    res <- apply(col2rgb(col), 2, function(c) rgb(c[1]/255, c[2]/255,c[3]/255, alpha))
    return(res)
  }
  # std vs residuals, QQplot (std vs teor quantiles), sqrt(std residuals) vs fitted, std res vs leverage = cook's distance
  nn <- 1:length(x) # number of fsa files read
  #par(mfrow=lay)
  
  layout(matrix(1:(lay[1]*lay[2]),lay[1],lay[2]))
  for(o in nn){
    prov <- x[[o]]
    
    if(is.null(color)){
      color <- transp(c("cornflowerblue", "chartreuse4", "gold2", "red", "orange", "purple",rainbow(10)))
    }else{color <- rep(color,10)}
    if(is.null(lims)){
    lims <- c(min(prov,na.rm = TRUE),max(prov,na.rm = TRUE))
    }
    uu <- 1:ncol(prov)
    if(!is.null(channel)){uu <- channel}
    
    for(u in uu){
      if(u == uu[1]){
        plot(prov[,u], type="l", col=color[u], las=2, ylab="Intensity", ylim = lims, main=names(x)[o], ...)
      }else{
        lines(x[[o]][,u], col=color[u])
      }
      
    }
  }
  plot(1,pch=16, col="white", xlab = "", ylab = "", xaxt="n", yaxt="n" )
  
  
  legend("center",col = color[uu], lwd=3, cex = cex.legend, ncol=ncol.legend,
         legend = paste("channel_",uu))
  #####################
  layout(matrix(1,1,1))
}

\name{my.plants}
\alias{my.plants}
\docType{data}
\title{
Cranberry biparental population
}
\description{
This dataset are 60 individuals from a progeny coming from the cross of 2 cranberry plants. Six SSR markers were run, 2 in the first channel (blue), 2 in the second channel (green), 2 in the third channel (yellow) and the Roxtrash375 ladder was run in the fourth channel (red). 
}
\usage{data("my.plants")}
\format{
  The format is:
 chr "my.plants"
}
\details{
The data is basically the raw FSA files coming from the ABi machine. No more details for this data.
}
\source{
This data was generated by the Cranberry Genomics Lab.
}
\references{
Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragma: An R package for fragment analysis. http://horticulture.wisc.edu/cggl/ZalapaLab/People.html. 2015.
}
\examples{
data(my.plants)
## look at the list structure
str(my.plants) 
}
\keyword{datasets}

\name{saturate}
\alias{saturate}
\title{Checking and correcting saturated peaks}

\description{
This function takes a vector of intensities and looks for peaks above 8000 RFUs and correct for possible splits at the top of the peaks by inverting the vally between splitted peaks and correcting the peak. 
}
\usage{
saturate(y)
}

\arguments{
  \item{y}{ a vector containing the DNA intensities for the capillary electrophoresis.}

}
\details{
No major details.
}
\value{
If arguments are correctly specified the function returns:
\describe{
\item{$y}{ A new vector of DNA intensities adjusted for saturated peaks over 8000 RFUs.}
}
}

\references{
Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
data(my.plants)
y <- my.plants[[1]][,3]
layout(matrix(1:2,2,1))
plot(y, type="l", xlim=c(2750,2850))
y2 <- saturate(y=y)
plot(y2, type="l", xlim=c(2750,2850))
}

\name{reals}
\alias{reals}
\title{Finding the real peaks}

\description{
This function takes a list with the information of positions, heights and weights for an individual and using the panel information finds the real peaks by using the separate function and getting the tallest peaks in the confidence interval constructed for the heights in the inteval of interest.
}
\usage{
reals(x, panel=c(100:400), shi=1, ploidy=2, left.cond=c(0.4,3), 
      right.cond=0.2, window=0.5)
}

\arguments{
  \item{x}{List with 3 elements; the information of positions, heights and weights for an individual in certain channel}

\item{panel}{A vector containing the base pair interval where the peaks should be searched for}

\item{shi}{The number of base pairs to be used for discarding neighboring peaks to the tallest peaks, i.e. if 2 peaks are 0.3 bp together the smalles will be discarded}

\item{ploidy}{A scalar value indicating the ploidy of the organism to be scored}

\item{left.cond}{A percentage value indicating when peaks to the leaft of the tallest peaks should be considered real based on the height, i.e. a very close peak right before the tallest peak if smaller than the tallest (half the size of the tallest one will be real or not)}

\item{right.cond}{A percentage value indicating when peaks to the right of the tallest peaks should be considered real based on the height, i.e. a very close peak right after the tallest peak if smaller than the tallest (half the size of the tallest one will be real or not)}

\item{window}{A value in base pairs indicating how much is the error for detecting a peak in a sample that was provided in the panelas a real peak}
}
\details{

No major details.
}
\value{
If arguments are correct the function returns a list containing
\describe{
\item{$pos}{the index positions for the intensities}
\item{$hei}{the intensities for the fragments found}
\item{$wei}{the putative weights in base pairs based on the ladder provided}
}
}

\references{
Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
data(my.plants)
x <- big.peaks.col(my.plants[[1]][,1],100)#for any color
#reals(x, panel=c(260,280), shi=1, ploidy=2) 
#still needs weight information in order to find the reals, 
#works internally of score.easy function
}

\name{plot.fsa_stored}
\alias{plot.fsa_stored}
\title{plot form fsa files stored with storing.inds}
\description{\code{plot} method for class \code{"fsa_stored"}.
}
\usage{
\method{plot}{fsa_stored}(x, lay=c(2,1), channel=NULL, cex.legend=.5, ncol.legend=4,lims=NULL, color=NULL, \dots)
}
\arguments{
 \item{x}{an object of class \code{"fsa_stored"}}
 \item{lay}{layout for the numberof plots to visualize.}
 \item{channel}{if preferred a single channel can be specified.}
 \item{cex.legend}{value of the size of the legend.}
 \item{ncol.legend}{number of columns to divide the legend.}
 \item{lims}{equivalen to ylim argument in plot, xlim still can be used.}
 \item{color}{specific colors to use in case the user don't want the default palete.}
 \item{\dots}{Further arguments to be passed}
}
\value{
  vector of plot
 }
\author{Giovanny Covarrubias \email{covarrubiasp@wisc.edu}}
\seealso{\code{\link{Fragman}}}
\keyword{models}

\name{letter.to.jm}
\alias{letter.to.jm}
\title{
Letter to JoinMap code converter
}
\description{
This function converts a vector of ssr calls in letter format to joinmap code using information from mother and father provided in first and second row respectively
}
\usage{
letter.to.jm(x)
}

\arguments{
  \item{x}{A vector of ssr calls in letter format or snp types, mother and father of the population should be in 1st and 2nd position respectively}
}
\details{

If numeric data exists first needs to be converted to letter code in order to use this function.
}
\value{
If arguments are correct the function returns a list containing
\describe{
\item{$y}{ A vector with ssr calls in joinmap format}
}
}

\references{
Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
xx <- data.frame(cbind(a=rep(150, 96), b=c(rep(100,48), rep(150,48))))
xx1 <- num.to.lett(xx)
letter.to.jm(unlist(xx1))
# try using apply to a dataframe
}

\name{score.markers}
\alias{score.markers}
\title{Fragment analysis scoring}

\description{
This function uses information from the fsa files read from \code{\link{storing.inds}} function and does the ssr calling in the channel specified and returns the index position, height and base pair position.

}
\usage{
score.markers(my.inds, channel = 1, n.inds = NULL, panel=NULL, shift=0.8,
          ladder, channel.ladder=NULL, 
          ploidy=2, left.cond=c(0.6,3), right.cond=0.35, warn=FALSE, 
          window=0.5, init.thresh=200, ladd.init.thresh=200, 
          method="iter2", env = parent.frame(), my.palette=NULL,
          plotting=TRUE,  electro=FALSE, pref=3)
}

\arguments{
  \item{my.inds}{List with the channels information from the individuals specified, usually coming from the \code{\link{storing.inds}} function output}

\item{channel}{The channel you wish to analyze, usually 1 is blue, 2 is green, 3 is yellow, 4 is red and so on}

\item{n.inds}{Vector specifying the plants to be scored}

\item{panel}{A vector containing the base pair interval where the peaks should be searched for}

\item{shift}{The number of base pairs to be used for discarding neighboring peaks to the tallest peaks, i.e. if 2 peaks are 0.3 bp together the smalles will be discarded}

\item{ladder}{A vector containing the expected weights for the ladder peaks that will be found the using the \code{\link{find.ladder}} function}

\item{channel.ladder}{A scalar value indicating in which channel or color the ladder was read}

\item{ploidy}{A scalar value indicating the ploidy of the organism to be scored to decide the maximum number of peaks the program should look for. TO BE IMPLEMENTED SOON. STILL NOT FUNCTIONAL.}

\item{left.cond}{A percentage value (0-1) indicating when peaks to the left of the tallest peaks should be considered real based on the height, i.e. a value of 0.5 would mean that a close peak (to the left of the tallest peak) will be picked only if such peak is at least 50 percent as tall with respect to the tallest peak. The second argument is the number of base pair indicating when peaks to the left of the tallest peaks should be considered real based on the distance, i.e. a value of 3 would mean that a close peak (to the left of the tallest peak) will be picked only if such peak is at least 3 base pairs far away from the tallest peak}

\item{right.cond}{A percentage value (0-1) indicating when peaks to the right of the tallest peaks should be considered real based on the height, i.e. a value of 0.5 would mean that a close peak (to the right of the tallest peak) will be picked only if such peak is at least 50 percent as tall with respect to the tallest peak.}

\item{warn}{A TRUE/FALSE value indicating if warnings should be provided when detecting the ladder}

\item{window}{A value in base pairs indicating how much is the error for detecting a peak in a sample when providing a panel with expected peaks.}

\item{init.thresh}{ An initial value of intensity to detect peaks. We recommend not to deal to much with it unless you have highly controlled dna concentrations in your experiment.}

\item{ladd.init.thresh}{ If samples were not sized using the info.ladder.attach function this value will be used to detect ladder peaks. Internally the program will use the \code{\link{find.ladder}} function. We recommend not to deal to much with it unless you identified special situations with your ladder}

\item{method}{If samples were not sized using the info.ladder.attach function this method will be used to detect ladder peaks. An argument indicating one of the 3 methods available; "cor" makes all possible combination of peaks and searches exhaustive correlations to find the right peaks corresponsding to the expected DNA weights, or "ci" constructing confidence intervals to look for peaks meeting the conditions specified in the previous arguments, "iter2" an iterative procedure looking for the most likely peaks meeting your ladder expectation. Default is "iter2".}

\item{env}{ this is used to detect the environment of the user and load the result in the same environment. Don't mess with it please.}

\item{my.palette}{A character vector with the colors to be used when drawing the RFU plots. If NULL it will use the programmed palette.}

\item{plotting}{ a TRUE/FALSE value indicating if the plots should be drawn or not. The default value is TRUE.}

\item{electro}{A TRUE/FALSE value indicating if the electrogram/gel should be drawn or not. The default value is FALSE.}

\item{pref}{A scalar value indicating how many plots should be drawn in the output plotting. The dafault is 3.}
}
\details{

Method "ci" has been depreciated, currently the method "iter2" is the default and uses the ladder provided and observed peaks to match them using an iterative procedure based on least squares.
}
\value{
If arguments are correct the function returns a plot and a list containing
\describe{
\item{$pos}{the index positions for the intensities}
\item{$hei}{the intensities for the fragments found}
\item{$wei}{the putative weights in base pairs based on the ladder provided}
}
}

\references{

We have spent valuable time developing this package, please cite it in your publication:

Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
## ================================= ##
## ================================= ##
## Fragment analysis requires 
## 1) loading your data
## 2) matching your ladder
## 3) define a panel for scoring
## 4) score the samples
## ================================= ##
## ================================= ##

#####################
## 1) Load your data
#####################

### you would use something like:
# folder <- "~/myfolder"
# my.plants <- storing.inds(folder)
### here we just load our sample data and use the first 2 plants

?my.plants
data(my.plants)
my.plants <- my.plants[1:2]
class(my.plants) <- "fsa_stored"

#######################
## 2) Match your ladder
#######################

### create a vector indicating the sizes of your ladder and do the match

my.ladder <- c(50, 75, 100, 125, 129, 150, 175, 200, 225, 250, 275, 300, 325, 350, 375)
ladder.info.attach(stored=my.plants, ladder=my.ladder)

### matching your ladder is a critical step and should only happen once per batch of 
### samples read

###****************************************************************************************###
### OPTIONAL:
### If the ladder.info attach function detects some bad samples 
### that you can correct them manually using
### the ladder.corrector() function
### For example to correct one sample in the previous data
### ladder.corrector(stored=my.plants, 
#to.correct="FHN152-CPN01_01A_GH1x35_152-148-209_717-704-793_367-382-381.fsa", 
#ladder=my.ladder)
###****************************************************************************************###

#######################
## 3) Define a panel
#######################

### In fragment analysis you usually design a panel where you indicate
### which peaks are real. You may use the overview2 function which plots all the
### plants in the channel you want in the base pair range you want

overview2(my.inds=my.plants, channel = 2:3, ladder=my.ladder, init.thresh=5000)

### You can click on the peaks you think are real, given that the ones
### suggested by the program may not be correct. This can be done by using the 
### 'locator' function and press 'Esc' when you're done, i.e.:
# my.panel <- locator(type="p", pch=20, col="red")$x
### That way you can click over the peaks and get the sizes
### in base pairs stored in a vector named my.panel

### Just for demonstration purposes I will use the suggested peaks by 
### the program using overview2, which will return a vector with 
### expected DNA sizes to be used in the next step for scoring
### we'll do it in the 160-190 bp region

my.panel <- overview2(my.inds=my.plants, channel = 3, 
                    ladder=my.ladder, init.thresh=7000, 
                    xlim=c(160,190)); my.panel

##########################
## 4) Score the samples
##########################

### When a panel is created is time to score the samples by providing the initial
### data we read, the ladder vector, the panel vector, and our specifications
### of channel to score (other arguments are available)

### Here we will score our samples for channel 3 with our panel created previously

res <- score.markers(my.inds=my.plants, channel = 3, panel=my.panel$channel_3,
                ladder=my.ladder, electro=FALSE)

### Check the plots and make sure they were scored correctly. In case some samples 
### are wrong you might want to use the locator function again and figure out 
### the size of your peaks. To extract your peaks in a data.frame do the following:

final.results <- get.scores(res)
final.results 
}

\name{jm.conv}
\alias{jm.conv}
\title{
Scores to JoinMap converter
}
\description{
This function converts a data frame containing the scores from score.easy to a new data frame with joinmap calls. The parents need to be provided in the first and second row. Currently only works for CP type of crosses. From CP type to F2 and BC2 is straight forward.
}
\usage{
jm.conv(a)
}

\arguments{
  \item{a}{A data frame containing the scores from score.easy function extracted from get.scores function containing the parental calls in the first 2 rows.
  }
}
\details{

No major details.
}
\value{
If arguments are correct the function returns a new data frame
\describe{
\item{res}{ A new data frame with markers in joinmap format}
}
}

\references{
Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
xx <- data.frame(cbind(a=rep(150, 96), b=c(rep(100,48), rep(150,48))))
jm.conv(xx)
# try using apply to a dataframe
}

\name{lapply_pb}
\alias{lapply_pb}
\title{complementary tools for Fragman}
\description{
This function is a wrapper of lapply function that allows the drawing of a progress bar to assess the speed of the process.
}
\usage{
lapply_pb (X, FUN, ...)
}
\arguments{
\item{X}{a vector (atomic or list) or an expression object. See see \code{\link{lapply}}.}
\item{FUN}{the function to be applied to each element of X: see \code{\link{lapply}}.}
\item{...}{passes another arguments to the typical lapply function.}

}
\details{
No major details
}
\value{
Performs lapply drawing a progress bar
\describe{
\item{res}{the same result than using lapply}
}
}

\references{
See see \code{\link{lapply}}
}

\examples{
l <- sapply(1:200, function(x) list(rnorm(1000)))
lapply_pb(l, mean)
}

\name{pullup}
\alias{pullup}
\title{Applying pullup to channels/colors}

\description{
This function takes a matrix of DNA intensities and merge all the channels (columns) to identify overall peaks and then creates a window moving from peak to peak looking for the channel where this peak is real and adjust the intensities in the other channels. 
}
\usage{
pullup(mati, plotting=FALSE, channel=4)
}

\arguments{
  \item{mati}{ matrix of intensities where each column is a channel/color for a given sample.}

\item{plotting}{ a TRUE/FALSE value indicating if the results from adjusting the intensities should be drawn or not.}
  
\item{channel}{ a numeric value indicating which of the channles/color (column) allocates the ladder intensties.}
}
\details{
No major details.
}
\value{
If arguments are correctly specified the function returns:
\describe{
\item{mati}{ A new matrix of DNA intensities corrected for overlapping of wavelenth readings in different channels.}
}
}

\references{
Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
data(my.plants)
layout(matrix(1:2,2,1))
# without pull up adjustment
plot(my.plants[[1]][,1], type="l", col="blue", xlim=c(2750,2850))
lines(my.plants[[1]][,2], col="green")
lines(my.plants[[1]][,3], col="gold")
## adjusted
yy <- pullup(my.plants[[1]])
plot(yy[,1], type="l", col="blue", xlim=c(2750,2850))
lines(yy[,2], col="green")
lines(yy[,3], col="gold")
# general view
yy1 <- pullup(my.plants[[1]], plotting=TRUE)
}

\name{get.scores}
\alias{get.scores}
\title{complementary tools}

\description{
This function extracts the information from auto.score and score.easy functions and fits everything in a dataframe.
}
\usage{
get.scores(my.scores, mark = "mark")
}

\arguments{
  \item{my.scores}{List of individuals which contains at the same time a list of positions, heights and weights for the fragment analysis}

\item{mark}{A vector of names of the markers scored in the panel or the parents}
}
\details{
Nomajor details.
}
\value{
If arguments are correct the function returns a data frame containing
\describe{
\item{$da}{A dataframe containing the ssr calls}
}
}

\references{
Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{

## here "a" is a similar ouptput to the `score.easy` function
par1 <- list(pos=c(3100, 3240), hei=c(22917,20563), wei=c(202,212))
par2 <- list(pos=c(3100, 3240), hei=c(22917,20563), wei=c(202,214))
a <- list(i1=par1, i2=par2)
get.scores(a)
}

\name{arrange.jm}
\alias{arrange.jm}
\title{
Arrange data converted to joinmap code into a joinmap readable file 
}
\description{
This function converts a data frame containing the joinmap code into the readable file for joinmap. This format still needs some extra information, specifically the header indicating the population type, no.loci and no. of individuals, please check file examples included in JoinMap software. 
}
\usage{
arrange.jm(x, par=FALSE) 
}

\arguments{
  \item{x}{A data frame containing the scores in jm coding.
  }
  
  \item{par}{A TRUE/FALSE value indicating if the data returned should include the parents or not, the dafault value is FALSE, indicating that the first 2 individuals will not be included.
  }
}
\details{

No major details.
}
\value{
If arguments are correct the function returns a new data frame
\describe{
\item{joinmap}{ A new data frame with markers in joinmap readable format}
}
}

\references{
Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
xx <- data.frame(cbind(a=rep(150, 96), b=c(rep(100,48), rep(150,48)))); xx[1,] <- c(150,150)
xx2 <- cbind(jm.conv(xx), jm.conv(xx), jm.conv(xx))
xx3 <- arrange.jm(xx2, par=FALSE) 
xx3[,1:10]
}

\name{detect.ladder}
\alias{detect.ladder}
\title{Ladder detection by correlation or confidence intervals}

\description{
This function takes a vector of color heights/intensities from the fragment analysis containing the ladder/standard channel, and detects the biggest peaks where the derivative is equal zero and uses the information from the expected weights for the ladder to construct confidence intervals in order to detect the ladder peaks.

Please! if using the confidence interval method ("ci"), which is NOT the default,  once you have found the best parameters for the arguments to match your ladder using this function, please pass those values to all the posterior functions, making sure the 'dev' argument is passed to the new functions. If using the correlation method ("cor"), don't worry about it.
}
\usage{

detect.ladder(stored, ind=1, ladder, channel.ladder=dim(stored[[1]])[2],
              ci.upp=1.96, ci.low=1.96, draw=TRUE, dev=50, warn=TRUE, 
              init.thresh=250,sep.index=8, method="cor", avoid=1500, who="sample")

}

\arguments{
 \item{stored}{Lis of dataframes obtained by using the \code{\link{storing.inds}} function.}
 
\item{ind}{The individual that you wish to analyze for assessing that the ladder was correctly detected.}

\item{ladder}{Vector containing the expected weights of the dna fragments of the ladder in use}

\item{channel.ladder}{A scalar value indicating in which channel or color the ladder was read}

\item{ci.upp}{A scalar value indicating how many standar errors will be used to detect peaks when checking the height of the ladder peaks(upper bound). To be used in the \code{\link{find.ladder}} function}

\item{ci.low}{A scalar value indicating how many standar errors will be used to detect peaks when checking the height of the ladder peaks(lower bound). To be used in the 'find.ladder' function}

\item{draw}{A TRUE/FALSE value indicating if the plot for the ladder found should be printed or not}

\item{dev}{A scalar value indicating the number of indexes to be used as peak separation when deciding the ladder peaks. Some ladders contain dna fragments of very closed weights and modifying this parameter helps to detect them correctly}

\item{warn}{A TRUE/FALSE value indicating if warnings should be provided when detecting the ladder}

\item{init.thresh}{An initial value of color intensity to be used when detecting the ladder, could be really important for the correlation method}

\item{sep.index}{A scalar value indicating how many indexes should be allowed to considered a true peak from noisy peaks}

\item{method}{An argument indicating one of the 2 methods available; "cor" makes all possible combination of peaks and searches exhaustive correlations to find the right peaks corresponsding to the expected DNA weights, or "ci" constructing confidence intervals to look for peaks meeting the conditions specified in the previous arguments}

\item{who}{A name to indicate which sample is being analyzed}

\item{avoid}{A scalar value indicating how many indexes should be avoided when the method of correlation fails to find peaks and a random sample will be drawn from the existing peaks. The default is 1500 indexes which will samples peaks avoiding the first 1500 indexes which is usually related to noisy area in some ladders.}
}
\details{
The peaks are detected by default using a correlation method bu the user can use confidence intervals if desired.
}
\value{
If parameters are indicated correctly the function returns:
\describe{
\item{$pos}{the index positions for the intensities}
\item{$hei}{the intensities for the fragments found}
\item{$wei}{the putative weights in base pairs based on the ladder provided}
}
}

\references{

Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
data(my.plants)
my.ladder <- c(120, 125, 129, 150, 175, 200, 225, 250, 275, 300, 325, 350, 375)
# looking at the first individual
detect.ladder(stored=my.plants, ind=1, ladder=my.ladder)
}

\name{storing.inds}
\alias{storing.inds}
\title{Extracting channel information}

\description{
This function reads the FSA files using a function named 'read.abif' from another R package called seqinr. This will extract the information of the DNA intensities of the capillary electrophoresis and will store it in a data structure know in R as a list. The usage of the function and the arguments it takes are as follows:
}
\usage{
storing.inds(folder, channels=NULL, fourier=TRUE, 
            saturated=TRUE, lets.pullup=TRUE, 
            plotting=FALSE, rawPlot=FALSE,
            llength=3000, ulength=80000)
}

\arguments{
  \item{folder}{A path/directory where the FSA files are located. We recommend to use the Seession tab -> Set working directory and provide that folder as an argument}

\item{channels}{A scalar value indicating how many channels/colors should be found in the FSA files, usually people using the rox375 ladder in the red channel have 4 channels, people using the LIZ ladder have 5 channels. The default is the last channel.}

\item{fourier}{A FALSE/TRUE value indicating if data should be smooth aplying a Fourier transormation using 40 percent of the lowest frequencies. The dafault is TRUE}

\item{saturated}{A FALSE/TRUE value indicating if data should be checked and treated for saturated peaks above 8000 RFU which usually split at the top in 2 different peaks. The dafault is TRUE}

\item{lets.pullup}{A FALSE/TRUE value indicating if data should be treated for noise from channel to channel known as pull up or pull down peaks since wavelengths where the dyes are read usually overlap (blue->green->yellow->red->orange. The dafault is TRUE}

\item{plotting}{A FALSE/TRUE value indicating if results after data cleaning steps should be plotted to asses graphically how data was handled. The dafault is FALSE}

\item{rawPlot}{A FALSE/TRUE value indicating if a plot drawing all vectors read should be plotted. The dafault is FALSE since this consumes a lot of memory.}

\item{llength}{A numeric value indicating how small can be the number of indexes in each channel. Default is 3000 indexes for small runs.}

\item{ulength}{A numeric value indicating how big can be the number of indexes in each channel. Default is 80000 indexes for long runs.}

}
\details{
No major details.
}
\value{
If arguments are correct the function returns a list containing
\describe{
\item{all.inds.mats}{A list where each element is a data frame containing the "n" channels of an individual}
}
}

\references{

We have spent valuable time developing this package, please cite it in your publication:

Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
data(my.plants)
### the correct way to do it for a population of inds with 
### 4 colors + ladder= 5 would be:
# my.plants <- storing.inds(folder)
}

\name{transp}
\alias{transp}
\title{Creating color with transparency}

\description{
This function takes a color and returns the same with a certain alpha grade transparency.
}
\usage{
transp(col, alpha=0.5)
}

\arguments{
  \item{col}{Color to be used for transparency}

\item{alpha}{Grade of transparency desired}
}
\details{
No major details.
}
\value{
If arguments are correctly specified the function returns:
\describe{
\item{$res}{ A new color with certain grade of transparency}
}
}

\references{
Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
transp("red", alpha=0.5)
}

\name{ladder.info.attach}
\alias{ladder.info.attach}
\title{Ladder detection and attachment to R environment}

\description{
This function uses the information stored by the \code{\link{storing.inds}} function and a vector specifying the ladder/standard and finds the real peaks corresponding to the expected weights. The user may use this function to be able to load the ladder information in the global environment of R, so when using the \code{\link{overview}} or \code{\link{score.markers}} functions calculations will be performed faster, if the function is not used the program will calculate the ladder information each time \code{\link{overview}} or \code{\link{score.markers}} functions are used. 

NOTE: THE STEP OF MATCHING THE LADDER WITH YOUR SAMPLES USING THE `ladder.info.attach` FUNCTION IS CRITICAL. IF YOU HAVE ANY PROBLEM TRY MODIFYING THE ARGUMENT 'method', WITH THE 2 MOST EFFECTIVE METHODS method="iter" OR method="iter2"

}
\usage{
ladder.info.attach(stored, ladder, channel.ladder=NULL, 
                  method="iter2", ladd.init.thresh=NULL, 
                  env = parent.frame(), prog=TRUE, 
                  draw=TRUE, attempt=10)
}

\arguments{
 \item{stored}{List with the channels information from the individuals specified, usually coming from the \code{\link{storing.inds}} function output.}

\item{ladder}{Vector containing the expected weights of the dna fragments of the ladder in use.}

\item{channel.ladder}{A scalar value indicating in which channel or color the ladder was read}

\item{method}{An argument indicating one of the methods available; "iter" makes an iterative procedure to find the ladder, "iter2" the same but backwards, "cor" makes all possible combination of peaks and searches exhaustive correlations to find the right peaks corresponsding to the expected DNA weights, or "ci" constructing confidence intervals to look for peaks meeting the conditions specified in the previous arguments. The default allows the program to pick among "iter" and "iter2". Older methods have been depreciated.}

\item{ladd.init.thresh}{ A value of intensity to detect peaks in the internal use of the \code{\link{find.ladder}} function. We recommend not to deal to much with it unless you identified special situations with your ladder}

\item{env}{ this is used to detect the environment of the user and load the result in the same environment.}

\item{prog}{ A TRUE/FALSE value indicating if a progress bar should be drawn while processing the samples in order to assess the time it takes to find the ladder. The dafault value is TRUE but usually this makes the process slower. Please feel free to set it equal FALSE if the number of samples is quite large and speed is a concern.}

\item{draw}{ A TRUE/FALSE value indicating if a plot showing the peaks matched with your ladder should be drawn. The dafault value is FALSE to avoid extra delay. Please feel free to set it equal TRUE if you prefer to assess the sizing process.}

\item{attempt}{A scalar value indicating how many attempts should be made to find the real ladder peaks when using the "iter" and "iter2" methods. By default is 7 attempts, which means that will try to build the model assuming that the first peak found in the ladder is the corresponding first peak of the expected ladder, then moves to the 2nd peak until the 7th and the seven models are compared picking the most likely model based on the R2 value for each of the models.}

}
\details{

Method "ci" has been depreciated, currently the method "iter2" is the default and uses the ladder provided and observed peaks to match them using an iterative procedure based on least squares.

}
\value{
If parameters are indicated correctly the function returns:
\describe{
\item{$pos}{the index positions for the intensities}
\item{$hei}{the intensities for the fragments found}
\item{$wei}{the putative weights in base pairs based on the ladder provided}
}
}

\references{

We have spent valuable time developing this package, please cite it in your publication:

Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
data(my.plants)
my.plants <- my.plants[1:2]
my.ladder <- c(50, 75, 100, 125, 129, 150, 175, 200, 225, 250, 275, 300, 325, 350, 375)
ladder.info.attach(stored=my.plants, ladder=my.ladder)
}

\name{homogenize.to.parentals}
\alias{homogenize.to.parentals}
\title{complementary tools}

\description{
This functions takes a list of positions, heights and weights for ssr calls of a certain plant and uses parental information and a window to homogenize to the parent calls.
}
\usage{
homogenize.to.parentals(x, parents, window)
}

\arguments{
  \item{x}{list of positions, heights and weights}

\item{parents}{different parental calls}

\item{window}{window in base pairs indicating when should be considered the same allele}
}
\details{

No major details.
}
\value{
If arguments are correct the function returns a list containing
\describe{
\item{$pos}{the index positions for the intensities}
\item{$hei}{the intensities for the fragments found}
\item{$wei}{the putative weights in base pairs based on the ladder provided}
}
}

\references{
Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
#No example provided, internally working to round to the closest parent ssr call.
x <- 1:10

}

\name{big.peaks.col}
\alias{big.peaks.col}
\title{Peak search by first derivatives}

\description{
This function find all peaks by taking the first derivative based on 'rle' function. Is used in different Fragma functions.
}
\usage{
big.peaks.col(x,tre)
}

\arguments{
 \item{x}{A vector of heights or intensities}

\item{tre}{A scalar value deciding when peaks will be ignored}
}
\details{
No major details.
}
\value{
Retuns the biggest peaks for a vector of intensities.
\describe{
\item{out}{ a vector of positions where the derivative is zero and therefore a peak was found}
}
}

\references{

Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
data(my.plants)
big.peaks.col(my.plants[[1]][,1],100)#for any color
}

\name{Fragman-package}
\alias{Fragman}
\docType{package}
\author{
Giovanny Covarrubias-Pazaran, Luis Diaz-Garcia, Brandon Schlautman, Walter Salazar, Juan Zalapa.
}
\title{Fragment analysis and automatic scoring}
\description{Fragman is a package designed for Fragment analysis and automatic scoring of biparental populations (such as F1, F2, BC types) and populations for diversity studies. The program is designed to read files with FSA extension (which stands for FASTA-type file and contains lectures for DNA fragments), and .txt files from Beckman CEQ 8000 system, and extract the DNA intensities from the channels/colors where they are located, based on ABi machine plattforms to perform sizing and allele scoring. 

The core of the package and the workflow of the fragment analysis rely in the following 4 functions; 

1) \code{\link{storing.inds}}(function in charge of reading the FSA or txt(CQS) files and storing them with a list structure)

2) \code{\link{ladder.info.attach}} (uses the information read from the FSA files and a vector containing the ladder information (DNA size of the fragments) and matches the peaks from the channel where the ladder was run with the DNA sizes for all samples. Then loads such information in the R environment for the use of posterior functions)

3) \code{\link{overview2}} (create friendly plots for any number of individuals specified  and can be used to design panels (\code{\link{overview2}}) for posterior automatic scoring (like licensed software does), or make manual scoring (\code{\link{overview}}) of individuals such as parents of biparental populations or diversity populations)

4) The \code{\link{score.markers}} (function score the alleles by finding the peaks provided in the panel (if provided), otherwise returns all peaks present in the channel). Thisfinal function can be automatized if several markers are located in the same channel by creating lists of panels taking advantage of R capabilities and data structures.

** Sometimes during the ladder sizing process some samples can go wrong for several reasons related to the sample quality (low intensity in ladder channel, extreme number of noisy peaks, etc.), because of that we have introduced \code{\link{ladder.corrector}} function which allows the user to correct the bad samples by clicking over the real peaks, by default the \code{\link{ladder.info.attach}} function returns the names of the samples that had a low correlation with the expected peaks.

When automatic scoring is not desired the function \code{\link{overview}} can be used for getting an interactive session and click over the peaks (using the \code{\link{locator}} function) in order to get the allele sizes.
}

\section{Contact}{

Feel free to contact us with questions and improvement suggestions at:

covarrubiasp@wis.edu
	
Just send a sample file with your question to recreate the issue or bug reported along with vector for your ladder.
}

\section{Citation}{

We have spent valuable time developing this package, please cite it in your publication:

Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.
}

\references{
Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}
\keyword{ package }
\seealso{
http://cggl.horticulture.wisc.edu/home-page/
}
\examples{
## ================================= ##
## ================================= ##
## Fragment analysis requires 
## 1) loading your data
## 2) matching your ladder
## 3) define a panel for scoring
## 4) score the samples
## ================================= ##
## ================================= ##

#####################
## 1) Load your data
#####################

### you would use something like:
# folder <- "~/myfolder"
# my.plants <- storing.inds(folder)
### here we just load our sample data and use the first 2 plants

?my.plants
data(my.plants)
my.plants <- my.plants[1:2]
class(my.plants) <- "fsa_stored"
# plot(my.plants) # to visualize the raw data

#######################
## 2) Match your ladder
#######################

### create a vector indicating the sizes of your ladder and do the match

my.ladder <- c(50, 75, 100, 125, 129, 150, 175, 200, 225, 250, 275, 300, 325, 350, 375)
ladder.info.attach(stored=my.plants, ladder=my.ladder)

### matching your ladder is a critical step and should only happen once per batch of 
### samples read

###****************************************************************************************###
### OPTIONAL:
### If the ladder.info attach function detects some bad samples 
### that you can correct them manually using
### the ladder.corrector() function
### For example to correct one sample in the previous data
### ladder.corrector(stored=my.plants, 
#to.correct="FHN152-CPN01_01A_GH1x35_152-148-209_717-704-793_367-382-381.fsa", 
#ladder=my.ladder)
###****************************************************************************************###

#######################
## 3) Define a panel
#######################

### In fragment analysis you usually design a panel where you indicate
### which peaks are real. You may use the overview2 function which plots all the
### plants in the channel you want in the base pair range you want

overview2(my.inds=my.plants, channel = 2:3, ladder=my.ladder, init.thresh=5000)

### You can click on the peaks you think are real, given that the ones
### suggested by the program may not be correct. This can be done by using the 
### 'locator' function and press 'Esc' when you're done, i.e.:
# my.panel <- locator(type="p", pch=20, col="red")$x
### That way you can click over the peaks and get the sizes
### in base pairs stored in a vector named my.panel

### Just for demonstration purposes I will use the suggested peaks by 
### the program using overview2, which will return a vector with 
### expected DNA sizes to be used in the next step for scoring
### we'll do it in the 160-190 bp region

my.panel <- overview2(my.inds=my.plants, channel = 3, 
                    ladder=my.ladder, init.thresh=7000, 
                    xlim=c(160,190)); my.panel

##########################
## 4) Score the samples
##########################

### When a panel is created is time to score the samples by providing the initial
### data we read, the ladder vector, the panel vector, and our specifications
### of channel to score (other arguments are available)

### Here we will score our samples for channel 3 with our panel created previously

res <- score.markers(my.inds=my.plants, channel = 3, panel=my.panel$channel_3,
                ladder=my.ladder, electro=FALSE)

### Check the plots and make sure they were scored correctly. In case some samples 
### are wrong you might want to use the locator function again and figure out 
### the size of your peaks. To extract your peaks in a data.frame do the following:

final.results <- get.scores(res)
final.results 
}

\name{best.layout}
\alias{best.layout}
\title{complementary tools for layout}
\description{
This function just find the best layout fit for a number of plots desired.
}
\usage{
best.layout(x)
}
\arguments{
\item{x}{A scalar value indicating the number of plots desired}
}
\details{
No major details
}
\value{
Returns the best layout
\describe{
\item{res}{the number of rows and columns giving the best fit}
}
}

\references{

Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
best.layout(9)
}

\name{threshs}
\alias{threshs}
\title{Customizing thresholds}

\description{
This function takes data contained in a list with 2 elements, the first containing the position in base pairs and the heights of the positions and given a panel finds the best minimum threshold by creating a confidence interval in order to get only the real peaks. Implemented internally of auto,score and score.easy function.
}
\usage{
threshs(my.plant, min.thre=200, panel, ci=1.9)
}

\arguments{
  \item{my.plant}{List with 2 elements; the position in base pairs and the heights associated with them}

\item{min.thre}{Starting threshold to find the peaks and refine the search}

\item{panel}{Two scalar values indicatin the interval where the peaks should be found}

\item{ci}{Scalar value indicating the number of standard errors to be used when creating the confidence interval for ssr calling}
}
\details{
No major details.
}
\value{
If arguments are correctly specified the function returns a list containing
\describe{
\item{$newthre}{ A scalar value indicating the new threshold to be used when calling the real peaks}
}
}

\references{
Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
data(my.plants)
# implemented internally of auto,score and score.easy function
# threshs(my.plant, min.thre=200, panel=(260,290), ci=1.9)
}

\name{separate}
\alias{separate}
\title{Separating peaks by a shift window}

\description{
This function takes a list with positions, heights and weights called "g" and using a shift in base pairs determines when 2 neighboring peaks should be considered only one by getting the tallest peak. For example two peaks found at 173 and 173.5 base pairs are unlikely to be 2 different peaks, therefore only the tallest peak will pe chosen.
}
\usage{
separate(g, shift=1, type="bp")
}

\arguments{
  \item{g}{List with 3 elements; the information of positions, heights and weights for an individual in certain channel}

\item{shift}{The number of base pairs to be used for discarding neighboring peaks to the tallest peaks, i.e. if 2 peaks are 0.3 bp together the smalles will be discarded}

\item{type}{A word indicating if the shift to be used should be used in base pairs or in index. The use is "bp" or "ind"}
}
\details{

No major details.
}
\value{
If arguments are correct the function returns a new list containing
\describe{
\item{$pos}{the index positions for the intensities}
\item{$hei}{the intensities for the fragments found}
\item{$wei}{the putative weights in base pairs based on the ladder provided}
}
}

\references{
Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
data(my.plants)
x <- big.peaks.col(my.plants[[1]][,1],100)#for any color
#separate(x, shift=1, type="bp") #still needs weight information
}

\name{overview}
\alias{overview}
\title{Assesing several plants with an overview}

\description{
This function uses information from the FSA files read from \code{\link{storing.inds}} function and creates a plot to assess graphically the peaks of several plants in certain channel in order to score manually or assess the parental fragments in the case of biparentla ppulations. If you desire to create a panel you may want to take a look at \code{\link{overview2}}. The function contains several defaults in most of the arguments, please check arguments but in general.

}
\usage{
overview(my.inds, channel = 1, n.inds = c(1:length(my.inds)), 
        xlimi=c(min(ladder),max(ladder)), ladder, channel.ladder=dim(my.inds[[1]])[2], 
        ploidy=2, dev=50, method="iter", 
        init.thresh=200, ladd.init.thresh=200, warn=TRUE, my.palette=NULL,
        env = parent.frame())
}

\arguments{
  \item{my.inds}{List with the channels information from the individuals specified, usually coming from the \code{\link{storing.inds}} function output}

\item{channel}{The channel you wish to analyze, usually 1 is blue, 2 is green, 3 is yellow, 4 is red and so on}

\item{n.inds}{Vector specifying the plants to be scored}

\item{xlimi}{A vector containing the base pair interval where the plot should be drawn}

\item{ladder}{A vector containing the expected weights for the ladder peaks that will be found the using the \code{\link{find.ladder}} function}

\item{channel.ladder}{A scalar value indicating in which channel or color the ladder was read}

\item{ploidy}{A scalar value indicating the ploidy of the organism to be scored}

\item{dev}{A scalar value indicating the number of indexes to be used as peak separation when deciding the ladder peaks, for more details check \code{\link{find.ladder}} function}

\item{method}{An argument indicating one of the 2 methods available; "cor" makes all possible combination of peaks and searches exhaustive correlations to find the right peaks corresponsding to the expected DNA weights, or "ci" constructing confidence intervals to look for peaks meeting the conditions specified in the previous arguments}

\item{init.thresh}{ An initial value of intensity to detect peaks. We recommend not to deal to much with unless you have highly controlled dna concentrations in your experiment}

\item{ladd.init.thresh}{ A value of intensity to detect peaks in the internal use of the \code{\link{find.ladder}} function. We recommend not to deal to much with it unless you identified special situations with your ladder}

\item{warn}{A TRUE/FALSE value indicating if warnings should be provided when detecting the ladder}

\item{my.palette}{A character vector with the colors to be used when drawing the RFU plots. If NULL it will use the programmed palette.}

\item{env}{ this is used to detect the environment of the user and load the result in the same environment.}
}
\details{
No major details.
}
\value{
If rarguments are correct the function returns a list containing
\describe{
\item{$plot}{ Returns a plot joining the channel for the plants specified for the color desired and the peaks found by the function using the parameters specified}
  \item{$nana}{ Returns a vector with the names of the plants specified in the function}
}
}

\references{
Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
data(my.plants)
my.plants <- my.plants[1:10]
my.ladder <- c(50, 75, 100, 125, 129, 150, 175, 200, 225, 250, 275, 300, 325, 350, 375)
overview(my.inds=my.plants, channel = 1, n.inds = c(1:5), ladder=my.ladder, xlim=c(200,220))
# now use:
# locator(type="p", pch=20, col="red")$x
# to click over the peaks and get the sizes in base pairs
# when you are done make sure you press the "Esc" key, 
# do not push the stop button, some versions of R usually crash 
# by stopping instead of pressing 'Esc'.
}

\name{num.to.lett}
\alias{num.to.lett}
\title{
Number to Letter code converter
}
\description{
This function converts dataframes with rounded calls (numeric format in 2 cells), to letter format based on the GBS pipeline developed by Elshire et al. (2011) which can be used as intermediate step to transform to joinmap and Onemap formats, it requires mother and father in first and second row
}
\usage{
num.to.lett(xx)
}

\arguments{
  \item{xx}{matrix with numbers, every 2 columns is a marker and each row is an individual, parents are located in the first 2 rows}
}
\details{
No major details.
}
\value{
If arguments are correct the function returns a list containing
\describe{
\item{xx2}{ matrix coded in letter format where each column is a marker and each row is an individual}
}
}

\references{
Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
xx <- data.frame(cbind(a=rep(150, 96), b=c(rep(100,48), rep(150,48))))
num.to.lett(xx)
}

\name{transfft}
\alias{transfft}
\title{Applying the fourier transformation to a data frame}

\description{
This function takes a vector and applies a fourier transformation in order to smooth the peaks usinf the fft function in the base package. Use only top 40 percent of the lowest frequencies. 
}
\usage{
transfft(sn, top=0.3)
}

\arguments{
  \item{sn}{ a numeric vector containing the DNA intensities for the capillary electrophoresis.}

  \item{top}{ percent of lowest frequencies that should be used for the fourier transformation.}
}
\details{
No major details.
}
\value{
If arguments are correctly specified the function returns:
\describe{
\item{$y}{ A new vector of DNA intensities smoothed to avoid extra noisy peaks.}
}
}

\references{
Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
data(my.plants)
g1 <- transfft(my.plants[[1]][,4], top=0.8)
g2 <- transfft(my.plants[[1]][,4], top=0.4)
g3 <- transfft(my.plants[[1]][,4], top=0.1)
layout(matrix(1:3,3,1))
plot(g1, type="l")
lines(g2, col="red")
lines(g3, col="blue")
par1 <- c("top=0.8", "top=0.4", "top=0.1")
par2 <- c("black", "red", "blue")
par3 <- c(1,1,1)
legend("topright", legend=par1, col=par2, bty = "n", lty=par3, lwd=par3, cex=0.75)
}

\name{ladder.corrector}
\alias{ladder.corrector}
\title{Ladder corrector attached to R environment}

\description{
This function was designed to correct manually samples that could not be correctly detected by the \code{\link{ladder.info.attach}} function and allows the user to select manually the peaks he knows are the correct peaks. This function uses the output of the \code{\link{ladder.info.attach}} function which is basically a vector with the names of the samples that were too dificult for the algorithm to find. The console will draw a plot and will ask the user to click over the peaks expected for a ladder provided, once the user is done should press the 'esc' key and continue to the next sample. This process is repeated until the all samples with the names provided are adjusted. 

}
\usage{
ladder.corrector(stored, to.correct, ladder, 
      thresh=200, env = parent.frame(),...)
}

\arguments{
 \item{stored}{List with the channels information from the individuals specified, usually coming from the \code{\link{storing.inds}} function output.}
 
 \item{to.correct}{Vector containing the names ofthe samples to be corrected and usually the output of the \code{\link{ladder.info.attach}} function.}

\item{ladder}{Vector containing the expected weights of the dna fragments of the ladder in use.}

\item{thresh}{A scalar value indicating the minimum value in RFUs to look for peaks to match the user-selected peaks with the program recognized peaks.}

\item{env}{ this is used to detect the environment of the user and load the result in the same environment.}

\item{\dots}{Further arguments to be passed}

}
\details{
Once the user has selected the right peaks the function will fix the ladder and attach such information to the R environment.

}
\value{

This function does not produce any output.
\describe{
\item{$data}{the program will attach the information to the R environment}
}
}

\references{

We have spent valuable time developing this package, please cite it in your publication:

Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
data(my.plants)
my.plants <- my.plants[1:2]
my.ladder <- c(50, 75, 100, 125, 129, 150, 175, 200, 225, 250, 275, 300, 325, 350, 375)
ladder.info.attach(stored=my.plants, ladder=my.ladder, ladd.init.thresh=300)
## now if something goes wrong use the corrector:
#ladder.corrector(stored=my.plants, 
#to.correct="FHN152-CPN01_01A_GH1x35_152-148-209_717-704-793_367-382-381.fsa", 
#ladder=my.ladder)
}

\name{read.abif}
\alias{read.abif}
\title{Read ABIF formatted files}
\description{
ABIF stands for Applied Biosystem Inc. Format, a binary fromat modeled after TIFF format.
Corresponding files usually have an .ab1 or .fsa extension.
}
\usage{
read.abif(filename, max.bytes.in.file = file.info(filename)$size, 
 pied.de.pilote = 1.2, verbose = FALSE)
}
\arguments{
  \item{filename}{The name of the file.}
  \item{max.bytes.in.file}{The size in bytes of the file, defaulting to what is returned by file.info}	
  \item{pied.de.pilote}{Safety factor: the argument readBin is set as pied.de.pilote*max.bytes.in.file.}	
  \item{verbose}{logical [FALSE]. If TRUE verbose mode is on.}	
}
\details{
All data are imported into memory, there is no attempt to read items on the fly.
}
\value{
A list with three components: Header which is a list that contains various low-level information,
among which numelements is the number of elements in the directory and dataoffset
the offset to find the location of the directory. Directory is a data.frame for the directory
of the file with the number of row being the number of elements in the directory and the 7
columns describing various low-level information about the elements.  
}
\references{ 
Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Anonymous (2006) Applied Biosystem Genetic Analysis Data File Format.
Available at \url{http://www.appliedbiosystems.com/support/software_community/ABIF_File_Format.pdf}.
Last visited on 03-NOV-2008.

The figure in the example section is an attempt to reproduce figure 1A from:

Krawczyk, J.,  Goesmann, A., Nolte, R., Werber, M., Weisshaar, B. (2009)
Trace2PS and FSA2PS: two software toolkits for converting trace and fsa files to PostScript format.
\emph{Source Code for Biology and Medicine}, \bold{4}:4.

}
\author{J.R. Lobry}


\examples{
#No examples provided, please download seqinr package for going deeper in this function.

}

\name{overview2}
\alias{overview2}
\title{Assesing several plants with an overview}

\description{
This function uses information from the FSA files read from \code{\link{storing.inds}} function and creates an overlapping plot to assess graphically the peaks of several plants in certain channel in order to create a panel for the scoring functions \code{\link{score.markers}}. The function contains several defaults in most of the arguments, please check arguments but in general you only need the first 4 arguments to create a panel.

}
\usage{
overview2(my.inds, channel = 1, ladder, xlim = NULL, ylim = NULL, 
        n.inds = NULL, channel.ladder = NULL, ploidy = 2, 
        method="iter2", init.thresh=NULL, ladd.init.thresh=200, 
        lwd=.25, warn=TRUE, min.panel=100, suggested=TRUE, 
        env = parent.frame(), my.palette=NULL, verbose=TRUE) 

}

\arguments{
  \item{my.inds}{List with the channels information from the individuals specified, usually coming from the \code{\link{storing.inds}} function output}

\item{channel}{The channel/color you wish to analyze, usually 1 is blue, 2 is green, 3 is yellow, 4 is red and so on}

\item{ladder}{A vector containing the expected weights for the ladder peaks that will be found the using the \code{\link{find.ladder}} function}

\item{xlim}{A vector containing the base pair interval where the plot should be drawn}

\item{ylim}{A vector containing the intensity interval where the plot should be drawn}

\item{n.inds}{Vector specifying the plants to be scored}

\item{channel.ladder}{A scalar value indicating in which channel or color the ladder was read}

\item{ploidy}{A scalar value indicating the ploidy of the organism to be scored}

\item{method}{An argument indicating one of the 2 methods available; "cor" makes all possible combination of peaks and searches exhaustive correlations to find the right peaks corresponsding to the expected DNA weights, or "ci" constructing confidence intervals to look for peaks meeting the conditions specified in the previous arguments}

\item{init.thresh}{ An initial value of intensity to detect peaks. We recommend not to deal to much with unless you have highly controlled dna concentrations in your experiment}

\item{ladd.init.thresh}{ A value of intensity to detect peaks in the internal use of the \code{\link{find.ladder}} function. We recommend not to deal to much with it unless you identified special situations with your ladder}

\item{lwd}{ The width of the line}

\item{warn}{A TRUE/FALSE value indicating if warnings should be provided when detecting the ladder}

\item{min.panel}{A scalar value indicating which peak values should be ignored when creating a panel. If 'xlim' values are specified the 'min.panel' value is ignored and instead the panel peaks provided by the program are based in the region where you are zooming in.}

\item{suggested}{ a TRUE/FALSE value statement declaring if you want the program to return suggested peaks for your panel. The default is TRUE but can be anoying if the program draws too many peaks.}

\item{env}{ this is used to detect the environment of the user and load the result in the same environment.}

\item{my.palette}{A character vector with the colors to be used when drawing the RFU plots. If NULL it will use the programmed palette.}

\item{verbose}{A TRUE/FALSE statement indicating if the function should return informative messages.}

}
\details{
No major details.
}
\value{
If rarguments are correct the function returns a list containing
\describe{
\item{$plot}{ Returns a plot joining the channel for the plants specified for the color desired and the peaks found by the function using the parameters specified}
  \item{$nana}{ Returns a vector with the names of the plants specified in the function}
}
}

\references{

We have spent valuable time developing this package, please cite it in your publication:

Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
data(my.plants)
my.plants <- my.plants[1]
my.ladder <- c(50, 75, 100, 125, 129, 150, 175, 200, 225, 250, 275, 300, 325, 350, 375)
overview2(my.inds=my.plants, channel = 1, ladder=my.ladder, lwd=1)
# now use:
# my.panel <- locator(type="p", pch=20, col="red")$x
# to click over the peaks and get the sizes in base pairs
# when you are done make sure you press the "Esc" key, do not push the stop button

## to look at many channels at the same time you 
## can use the par(new=TRUE) and a for loop

for(u in 1:4){
  overview2(my.inds=my.plants, channel = u, ladder=my.ladder, lwd=1,
            xlim=c(240,350), ylim=c(0,30000))
  par(new=TRUE)
}

}

\name{find.ladder}
\alias{find.ladder}
\title{Ladder detection by correlation or confidence intervals}

\description{
This function takes a vector of color heights/intensities from the fragment analysis containing the ladder/standard channel, and detects the biggest peaks where the derivative is equal zero and uses the information from the expected weights for the ladder to construct confidence intervals in order to detect the ladder peaks.

Please! if using the confidence interval method ("ci"), which is not the default, once you have found the best parameters for the arguments to match your ladder using this function, please pass those values to all the posterior functions, please make sure the 'dev' argument is passed to the new functions.
}
\usage{
find.ladder(x, ladder, draw=TRUE, dev=50, warn=TRUE, init.thresh=NULL, 
            sep.index=8, method=NULL, reducing=NULL, who="sample", 
            attempt=10, cex.title=0.8)
}

\arguments{
 \item{x}{Vector of heights from the ladder channel. See example to see how to access to it.}

\item{ladder}{Vector containing the expected weights of the dna fragments of the ladder in use}

\item{draw}{A TRUE/FALSE value indicating if the plot for the ladder found should be printed or not}

\item{dev}{A scalar value indicating the number of indexes to be used as peak separation when deciding the ladder peaks. Some ladders contain dna fragments of very closed weights and modifying this parameter helps to detect them correctly}

\item{warn}{A TRUE/FALSE value indicating if warnings should be provided when detecting the ladder}

\item{init.thresh}{An initial value of color intensity to be used when detecting the ladder}

\item{sep.index}{A scalar value indicating how many indexes should be allowed to considered a true peak from noisy peaks}

\item{method}{An argument indicating one of the 2 methods available; "cor" makes all possible combination of peaks and searches exhaustive correlations to find the right peaks corresponsding to the expected DNA weights, or "ci" constructing confidence intervals to look for peaks meeting the conditions specified in the previous arguments}

\item{who}{A name to indicate which sample is being analyzed}

\item{attempt}{A scalar value indicating how many attempts should be made to find the real ladder peaks. By default is 7 attempts, which means that will try to build the model assuming that the first peak found in the ladder is the corresponding first peak of the expected ladder, then moves to the 2nd peak until the 7th and the seven models are compared picking the most likely model based on the R2 value for each of the models.}

\item{reducing}{A vector of values to reduce the search of peaks to certain indexes in the x axis. Default is NULL so it looks for all peaks for matching the ladder.}

\item{cex.title}{A scalar value indicating how big the title (name of the sample) in the plot should be.}

}
\details{
We have implemented 3 methods for sizing the ladder, each with their advantages and disadvantages. The default method named "red" which stands for "reduction" detect the region where peaks exist (in indexes) in the ladder channel and assumes that your ladder should have some equivalence in indexes and creates an 'expected ladder', then the putative ladder moves along the peak region and correlations and squared distances to the closest peaks are calculated. We have define the coefficient of similarity (CS) as cor(x,y)/var(z), where:

cor(x,y) are the correlations between expected and observed peaks, and
var(z)  is the sum of squares between the differences of expected and observed peaks.

This value usually let us identify the most likely peaks and then all possible combinations for those peaks are computed followed by exhaustive correlations of those combinations with the actual ladder. The highest correlation usually points to the right peaks, which is selected.

In addition the method "cor" is the previous version to "red" which doesn't reduce the search of peaks and computes all possible combinations of peaks from the beggining, with the drawback that slows down the detection process especially when the ladder intensities are low and noisy peaks exist in abundance.

The last method that has been superseded by the previous 2 is the "ci" method based on confidence intervals, which assumes that real ladder peaks have more or less the same intensity and a they can be found by finding the median intensity and computing a 90 percent confidence interval to find the rest of the peaks. This method has been proved to fail when the first condition is broken and ladder have real peaks with intensities greater than the expected.

}
\value{
If parameters are indicated correctly the function returns:
\describe{
\item{$pos}{the index positions for the intensities}
\item{$hei}{the intensities for the fragments found}
\item{$wei}{the putative weights in base pairs based on the ladder provided}
}
}

\references{
Covarrubias-Pazaran G, Diaz-Garcia L, Schlautman B, Salazar W, Zalapa J. Fragman: An R package for fragment analysis. 2016. BMC Genetics 17(62):1-8.

Robert J. Henry. 2013. Molecular Markers in Plants. Wiley-Blackwell. ISBN 978-0-470-95951-0.

Ben Hui Liu. 1998. Statistical Genomics. CRC Press LLC. ISBN 0-8493-3166-8.
}

\examples{
data(my.plants)
my.ladder <- c(50, 75, 100, 125, 129, 150, 175, 200, 225, 250, 275, 300, 325, 350, 375)
find.ladder(my.plants[[1]][,4], ladder=my.ladder)
}
